Package: CARDspa
Title: Spatially Informed Cell Type Deconvolution for Spatial
        Transcriptomics
Version: 1.4.0
Date: 2025-2-6
Authors@R: 
    c(person(given = "Ying", 
    family = "Ma", 
    email = "ying_ma@brown.edu",
    role = "aut"),
    person(given = "Jing", 
    family = "Fu", 
    email = "jing_fu@brown.edu",
    role = "cre"))
Description: CARD is a reference-based deconvolution method that estimates cell 
    type composition in spatial transcriptomics based on cell type specific 
    expression information obtained from a reference scRNA-seq data. A key 
    feature of CARD is its ability to accommodate spatial correlation in the 
    cell type composition across tissue locations, enabling accurate and 
    spatially informed cell type deconvolution as well as refined spatial map 
    construction. CARD relies on an efficient optimization algorithm for 
    constrained maximum likelihood estimation and is scalable to spatial 
    transcriptomics with tens of thousands of spatial locations and tens of 
    thousands of genes. 
License: GPL-3 + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
Depends: R (>= 4.3.0)
Imports: Rcpp (>= 1.0.7),RcppArmadillo, SummarizedExperiment, methods,
        MCMCpack, fields, wrMisc, concaveman, sp, dplyr, sf, Matrix,
        RANN, ggplot2, reshape2, RColorBrewer, S4Vectors, scatterpie,
        grDevices,ggcorrplot, stats, nnls, BiocParallel, NMF,
        spatstat.random, gtools, SingleCellExperiment,
        SpatialExperiment
LazyData: false
biocViews: Spatial, SingleCell, Transcriptomics, Visualization
LinkingTo: Rcpp, RcppArmadillo
Suggests: knitr, rmarkdown, testthat, RcppML, BiocStyle
VignetteBuilder: knitr
URL: https://github.com/YMa-lab/CARDspa
BugReports: https://github.com/YMa-lab/CARDspa/issues
git_url: https://git.bioconductor.org/packages/CARDspa
git_branch: RELEASE_3_23
git_last_commit: 67bc02b
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 21:39:31 UTC; biocbuild
Author: Ying Ma [aut],
  Jing Fu [cre]
Maintainer: Jing Fu <jing_fu@brown.edu>
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-29 04:35:54 UTC; unix

# Generated by roxygen2: do not edit by hand

export(CARD_deconvolution)
export(CARD_imputation)
export(CARD_refFree)
export(CARD_scmapping)
export(CARD_visualize_Cor)
export(CARD_visualize_gene)
export(CARD_visualize_pie)
export(CARD_visualize_prop)
export(CARD_visualize_prop_2CT)
import(Matrix)
import(RcppArmadillo)
import(SingleCellExperiment)
import(SpatialExperiment)
import(concaveman)
import(ggcorrplot)
import(ggplot2)
import(methods)
import(sp)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,SnowParam)
importFrom(BiocParallel,bplapply)
importFrom(MCMCpack,rdirichlet)
importFrom(RANN,nn2)
importFrom(RColorBrewer,brewer.pal)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,metadata)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SingleCellExperiment,colData)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(dplyr,'%>%')
importFrom(fields,rdist)
importFrom(grDevices,colorRampPalette)
importFrom(gtools,mixedsort)
importFrom(nnls,nnls)
importFrom(reshape2,melt)
importFrom(scatterpie,geom_scatterpie)
importFrom(sf,st_as_sf)
importFrom(spatstat.random,runifdisc)
importFrom(stats,aggregate)
importFrom(stats,cor)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,reshape)
importFrom(stats,runif)
importFrom(stats,var)
importFrom(wrMisc,rowGrpMeans)
importMethodsFrom(SummarizedExperiment,"colData<-")
useDynLib(CARDspa)

# CARDspa

## Spatially Informed Cell Type Deconvolution for Spatial Transcriptomics 
We developed CARDspa, a Bioconductor version of the original CARD method 
[https://github.com/YMa-lab/CARD], for spatially informed cell type 
deconvolution in spatial transcriptomics. CARDspa is a reference-based 
deconvolution tool that estimates the cell type composition in spatial 
transcriptomics data by utilizing cell-type-specific expression information 
obtained from reference single-cell RNA sequencing (scRNA-seq) data.

A key feature of CARDspa is its ability to accommodate spatial correlation in 
the cell type composition across different tissue locations, which ensures 
accurate and spatially informed deconvolution, as well as refined spatial map 
construction. This is achieved through an efficient optimization algorithm for 
constrained maximum likelihood estimation. CARDspa is scalable to handle spatial 
transcriptomics data with tens of thousands of spatial locations and genes.

As a Bioconductor package, CARDspa is designed to integrate seamlessly with 
other Bioconductor tools for analyzing and visualizing high-dimensional 
biological data. It leverages Bioconductor's robust framework for 
reproducibility and ease of use within the R ecosystem.

CARDspa is implemented as an open-source R package, and is freely available on 
Bioconductor at [https://bioconductor.org/packages/CARDspa/].

For further details, please visit our project page at 
[www.xzlab.org/software.html]. 

Installation
------------
You can install the released version of CARDspa using BioConductor.

``` r
# install BiocManager if necessary
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("CARDspa")

```

Or you can install using devtools in R.
``` r
# install devtools if necessary
install.packages('devtools')

# install the CARDspa package
devtools::install_github('https://github.com/YMa-lab/CARDspa')

# load package
library(CARDspa)

```

# Issues
All feedback, bug reports and suggestions are warmly welcomed! Please make sure 
to raise issues with a detailed and reproducible exmple and also please 
provide the output of your sessionInfo() in R! 

How to cite `CARDspa`
-------------------
Ma, Y., Zhou, X. Spatially informed cell-type deconvolution for spatial 
transcriptomics. Nat Biotechnol 40, 1349–1359 (2022). 
https://doi.org/10.1038/s41587-022-01273-7

How to use `CARDspa`
-------------------
Please see vignettes.
``` r
browseVignettes("CARDspa")
```

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)

## ----eval=FALSE---------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE)) {
#     install.packages("BiocManager")
# }
# BiocManager::install("CARDspa")

## -----------------------------------------------------------------------------
library(CARDspa)
library(RcppML)
library(NMF)
library(RcppArmadillo)
library(SingleCellExperiment)
library(SpatialExperiment)
library(ggplot2)
#### load the example spatial transcriptomics count data,
data(spatial_count)
spatial_count[1:4, 1:4]

## -----------------------------------------------------------------------------
#### load the example spatial location data,
data(spatial_location)
spatial_location[1:4, ]

## -----------------------------------------------------------------------------
data(sc_count)
sc_count[1:4, 1:4]

## -----------------------------------------------------------------------------
data(sc_meta)
sc_meta[1:4, ]

## -----------------------------------------------------------------------------
set.seed(seed = 20200107)
CARD_obj <- CARD_deconvolution(
    sc_count = sc_count,
    sc_meta = sc_meta,
    spatial_count = spatial_count,
    spatial_location = spatial_location,
    ct_varname = "cellType",
    ct_select = unique(sc_meta$cellType),
    sample_varname = "sampleInfo",
    mincountgene = 100,
    mincountspot = 5 )
## QC on scRNASeq dataset! ...
## QC on spatially-resolved dataset! ..
## create reference matrix from scRNASeq...
## Select Informative Genes! ...
## Deconvolution Starts! ...
## Deconvolution Finish! ...

## -----------------------------------------------------------------------------
## create sce object
sce <- SingleCellExperiment(assay = list(counts = sc_count),
                            colData = sc_meta)
## create spe object
spe <- SpatialExperiment(assay = list(counts = spatial_count),
                        spatialCoords  = as.matrix(spatial_location)
                        )
celltypes <- unique(sc_meta$cellType)
    
set.seed(seed = 20200107)
CARD_obj <- CARD_deconvolution(
    spe = spe,
    sce = sce,
    sc_count = NULL,
    sc_meta = NULL,
    spatial_count = NULL,
    spatial_location = NULL,
    ct_varname = "cellType",
    ct_select = celltypes,
    sample_varname = "sampleInfo",
    mincountgene = 100,
    mincountspot = 5
)

## -----------------------------------------------------------------------------
print(CARD_obj$Proportion_CARD[1:2, ])

## -----------------------------------------------------------------------------
## set the colors. Here, I just use the colors in the manuscript, if the color
## is not provided, the function will use default color in the package.
colors <- c(
    "#FFD92F", "#4DAF4A", "#FCCDE5", "#D9D9D9", "#377EB8", "#7FC97F",
    "#BEAED4", "#FDC086", "#FFFF99", "#386CB0", "#F0027F", "#BF5B17",
    "#666666", "#1B9E77", "#D95F02", "#7570B3", "#E7298A", "#66A61E",
    "#E6AB02", "#A6761D"
)
p1 <- CARD_visualize_pie(
    proportion = CARD_obj$Proportion_CARD,
    spatial_location = spatialCoords(CARD_obj),
    colors = colors,
    radius = 0.52
) ### You can choose radius = NULL or your own radius number
print(p1)

## -----------------------------------------------------------------------------
## select the cell type that we are interested
ct.visualize <- c(
    "Acinar_cells", "Cancer_clone_A", "Cancer_clone_B",
    "Ductal_terminal_ductal_like",
    "Ductal_CRISP3_high-centroacinar_like",
    "Ductal_MHC_Class_II",
    "Ductal_APOL1_high-hypoxic",
    "Fibroblasts"
)
## visualize the spatial distribution of the cell type proportion
p2 <- CARD_visualize_prop(
    proportion = CARD_obj$Proportion_CARD,
    spatial_location = spatialCoords(CARD_obj),
    ### selected cell types to visualize
    ct_visualize = ct.visualize,
    ### if not provide, we will use the default colors
    colors = c("lightblue", "lightyellow", "red"),
    ### number of columns in the figure panel
    NumCols = 4,
    ### point size in ggplot2 scatterplot
    pointSize = 3.0
)
print(p2)

## -----------------------------------------------------------------------------
## visualize the spatial distribution of two cell types on the same plot
p3 <- CARD_visualize_prop_2CT(
    ### Cell type proportion estimated by CARD
    proportion = CARD_obj$Proportion_CARD,
    ### spatial location information
    spatial_location = spatialCoords(CARD_obj),
    ### two cell types you want to visualize
    ct2_visualize = c("Cancer_clone_A", "Cancer_clone_B"),
    ### two color scales
    colors = list(
        c("lightblue", "lightyellow", "red"),
        c("lightblue", "lightyellow", "black")
    )
)
print(p3)

## -----------------------------------------------------------------------------
# if not provide, we will use the default colors
p4 <- CARD_visualize_Cor(CARD_obj$Proportion_CARD, colors = NULL)
print(p4)

## -----------------------------------------------------------------------------
CARD_obj <- CARD_imputation(
    CARD_obj, 
    num_grids = 2000, 
    ineibor = 10, 
    exclude = NULL)
## The rownames of locations are matched ...
## Make grids on new spatial locations ...

## -----------------------------------------------------------------------------
## Visualize the newly grided spatial locations to see if the shape is correctly
## detected. If not, the user can provide the row names of the excluded spatial
## location data into the CARD_imputation function
location_imputation <- cbind.data.frame(
    x = as.numeric(sapply(
        strsplit(rownames(CARD_obj$refined_prop), split = "x"), "[", 1
    )),
    y = as.numeric(sapply(
        strsplit(rownames(CARD_obj$refined_prop), split = "x"), "[", 2
    ))
)
rownames(location_imputation) <- rownames(CARD_obj$refined_prop)
library(ggplot2)
p5 <- ggplot(
    location_imputation,
    aes(x = x, y = y)
) +
    geom_point(shape = 22, color = "#7dc7f5") +
    theme(
        plot.margin = margin(0.1, 0.1, 0.1, 0.1, "cm"),
        legend.position = "bottom",
        panel.background = element_blank(),
        plot.background = element_blank(),
        panel.border = element_rect(colour = "grey89", fill = NA, 
                                    linewidth = 0.5)
    )
print(p5)

## -----------------------------------------------------------------------------
p6 <- CARD_visualize_prop(
    proportion = CARD_obj$refined_prop,
    spatial_location = location_imputation,
    ct_visualize = ct.visualize,
    colors = c("lightblue", "lightyellow", "red"),
    NumCols = 4
)
print(p6)

## -----------------------------------------------------------------------------
p7 <- CARD_visualize_gene(
    spatial_expression = assays(CARD_obj)$refined_expression,
    spatial_location = location_imputation,
    gene_visualize = c("A4GNT", "AAMDC", "CD248"),
    colors = NULL,
    NumCols = 6
)
print(p7)

## -----------------------------------------------------------------------------
p8 <- CARD_visualize_gene(
    spatial_expression = metadata(CARD_obj)$spatial_countMat,
    spatial_location = metadata(CARD_obj)$spatial_location,
    gene_visualize = c("A4GNT", "AAMDC", "CD248"),
    colors = NULL,
    NumCols = 6
)
print(p8)

## -----------------------------------------------------------------------------
## deconvolution using CARDfree
data(markerList)
set.seed(seed = 20200107)
CARDfree_obj <- CARD_refFree(
    markerlist = markerList,
    spatial_count = spatial_count,
    spatial_location = spatial_location,
    mincountgene = 100,
    mincountspot = 5
)

## -----------------------------------------------------------------------------
data(markerList)
set.seed(seed = 20200107)
CARDfree_obj <- CARD_refFree(
    markerlist = markerList,
    spatial_count = NULL,
    spatial_location = NULL,
    spe = spe,
    mincountgene = 100,
    mincountspot = 5
)

## -----------------------------------------------------------------------------
## One limitation of reference-free version of CARD is that the cell
## types inferred
## from CARDfree do not come with a cell type label. It might be difficult to
## interpret the results.
print(CARDfree_obj$Proportion_CARD[1:2, ])

## -----------------------------------------------------------------------------
colors <- c(
    "#FFD92F", "#4DAF4A", "#FCCDE5", "#D9D9D9", "#377EB8", "#7FC97F", "#BEAED4",
    "#FDC086", "#FFFF99", "#386CB0", "#F0027F", "#BF5B17", "#666666",
    "#1B9E77", "#D95F02", "#7570B3", "#E7298A", "#66A61E", "#E6AB02",
    "#A6761D"
)
### In order to maximumply match with the original results of CARD, we order the
### colors to generally match with the results infered by CARD
current_data <- CARDfree_obj$Proportion_CARD
new_order <- current_data[, c(
    8, 10, 14, 2, 1, 6, 12, 18, 7, 13, 20, 19, 16,
    17, 11, 15, 4, 9, 3, 5
)]
CARDfree_obj$Proportion_CARD <- new_order
colnames(CARDfree_obj$Proportion_CARD) <- paste0("CT", 1:20)
p9 <- CARD_visualize_pie(CARDfree_obj$Proportion_CARD,
    spatialCoords(CARDfree_obj),
    colors = colors
)
print(p9)

## -----------------------------------------------------------------------------
#### Note that here the shapeSpot is the user defined variable which
#### indicates the capturing area of single cells. Details see above.
set.seed(seed = 20210107)
scMapping <- CARD_scmapping(CARD_obj, shapeSpot = "Square", numcell = 20, 
                            ncore = 2)
print(scMapping)
### spatial location info and expression count of the single cell resolution
### data
MapCellCords <- as.data.frame(colData(scMapping))
count_SC <- assays(scMapping)$counts

## -----------------------------------------------------------------------------
df <- MapCellCords
colors <- c(
    "#8DD3C7", "#CFECBB", "#F4F4B9", "#CFCCCF", "#D1A7B9", "#E9D3DE", "#F4867C",
    "#C0979F", "#D5CFD6", "#86B1CD", "#CEB28B", "#EDBC63", "#C59CC5",
    "#C09CBF", "#C2D567", "#C9DAC3", "#E1EBA0",
    "#FFED6F", "#CDD796", "#F8CDDE"
)
p10 <- ggplot(df, aes(x = x, y = y, colour = CT)) +
    geom_point(size = 3.0) +
    scale_colour_manual(values = colors) +
    # facet_wrap(~Method,ncol = 2,nrow = 3) +
    theme(
        plot.margin = margin(0.1, 0.1, 0.1, 0.1, "cm"),
        panel.background = element_rect(colour = "white", fill = "white"),
        plot.background = element_rect(colour = "white", fill = "white"),
        legend.position = "bottom",
        panel.border = element_rect(
            colour = "grey89", 
            fill = NA, 
            linewidth = 0.5),
        axis.text = element_blank(),
        axis.ticks = element_blank(),
        axis.title = element_blank(),
        legend.title = element_text(size = 13, face = "bold"),
        legend.text = element_text(size = 12),
        legend.key = element_rect(colour = "transparent", fill = "white"),
        legend.key.size = unit(0.45, "cm"),
        strip.text = element_text(size = 15, face = "bold")
    ) +
    guides(color = guide_legend(title = "Cell Type"))
print(p10)

## -----------------------------------------------------------------------------
sessionInfo()

#' Construct the mean gene expression basis matrix (B), this is the
#' faster version
#'
#' @param sc_eset S4 class for storing data from single-cell experiments. 
#' This format is usually created by the package SingleCellExperiment with 
#' stored counts, along with the usual metadata for genes and cells.
#' @param ct_select vector of cell type names that you are interested in to
#' deconvolute, default as NULL. If NULL, then use all cell types provided by
#' single cell dataset;
#' @param ct_varname character, the name of the column in metaData that
#' specifies the cell type annotation information
#' @param sample_varname character,the name of the column in metaData that
#' specifies the sample information. If NULL, we just use the whole as one
#' sample.
#'
#' @importFrom SummarizedExperiment assays colData
#' @importFrom wrMisc rowGrpMeans
#' @importMethodsFrom SummarizedExperiment colData<-
#' @importFrom stats aggregate reshape
#' @return Return a list of basis (B) matrix
#'

create_ref <- function(
        sc_eset, 
        ct_select = NULL, 
        ct_varname,
        sample_varname = NULL) {
    ##### Prepare
    if (is.null(ct_select)) {
        ct_select <- unique(colData(sc_eset)[, ct_varname])
    }
    ct_select <- ct_select[!is.na(ct_select)]
    count_mat <- as(assays(sc_eset)$counts, "sparseMatrix")
    ct_id <- droplevels(as.factor(colData(sc_eset)[, ct_varname]))
    if (is.null(sample_varname)) {
        colData(sc_eset)$sampleID <- "Sample"
        sample_varname <- "sampleID"
    }
    sample_id <- as.character(colData(sc_eset)[, sample_varname])
    ct_sample_id <- paste(ct_id, sample_id, sep = "$*$")
    colsums_count_mat <- colSums(count_mat)
    colsums_count_mat_ct <- aggregate(
        colsums_count_mat ~ ct_id + sample_id,
        FUN = "sum"
    )
    colsums_count_mat_ct_wide <- reshape(
        colsums_count_mat_ct,
        idvar = "sample_id",
        timevar = "ct_id",
        direction = "wide"
    )
    colnames(colsums_count_mat_ct_wide) <-
        gsub("colsums_count_mat.", "", colnames(colsums_count_mat_ct_wide))
    rownames(colsums_count_mat_ct_wide) <- colsums_count_mat_ct_wide$sample_id
    colsums_count_mat_ct_wide$sample_id <- NULL
    tbl <- table(sample_id, ct_id)
    colsums_count_mat_ct_wide <-
        colsums_count_mat_ct_wide[,
            match(colnames(tbl),colnames(colsums_count_mat_ct_wide))]
    colsums_count_mat_ct_wide <-
        colsums_count_mat_ct_wide[
            match(rownames(tbl),rownames(colsums_count_mat_ct_wide)), ]
    
    ##### Calculate gene expression basis
    s_jk <- colsums_count_mat_ct_wide / tbl
    s_jk <- as.matrix(s_jk)
    s_jk[s_jk == 0] <- NA
    s_jk[!is.finite(s_jk)] <- NA
    S <- colMeans(s_jk, na.rm = TRUE)
    S <- S[match(unique(ct_id), names(S))]
    if (nrow(count_mat) > 10000 & ncol(count_mat) > 50000) {
        ### to save memory
        seqID <- seq(1, nrow(count_mat), by = 10000)
        theta_s_rowmean <- NULL
        for (igs in seqID) {
            if (igs != seqID[length(seqID)]) {
                theta_s_rowmean_tmp <- wrMisc::rowGrpMeans(
                    as.matrix(count_mat[(igs:(igs + 9999)), ]),
                    grp = ct_sample_id,
                    na.rm = TRUE
                )
            } else {
                theta_s_rowmean_tmp <- wrMisc::rowGrpMeans(
                    as.matrix(count_mat[igs:nrow(count_mat), ]),
                    grp = ct_sample_id,
                    na.rm = TRUE
                )
            }
            theta_s_rowmean <- rbind(theta_s_rowmean, theta_s_rowmean_tmp)
        }
    } else {
        theta_s_rowmean <- wrMisc::rowGrpMeans(as.matrix(count_mat),
            grp = ct_sample_id,
            na.rm = TRUE
        )
    }
    tbl_sample <- table(ct_sample_id)
    tbl_sample <- tbl_sample[match(
        colnames(theta_s_rowmean),
        names(tbl_sample))]
    theta_s_rowsums <- sweep(theta_s_rowmean, 2, tbl_sample, "*")
    theta_s <- sweep(theta_s_rowsums, 2, colSums(theta_s_rowsums), "/")
    grp <- vapply(
        strsplit(colnames(theta_s), split = "$*$", fixed = TRUE),
        `[`,
        FUN.VALUE = character(1),
        1
    )
    theta <- wrMisc::rowGrpMeans(theta_s, grp = grp, na.rm = TRUE)
    theta <- theta[, match(unique(ct_id), colnames(theta))]
    S <- S[match(colnames(theta), names(S))]
    basis <- sweep(theta, 2, S, "*")
    colnames(basis) <- colnames(theta)
    rownames(basis) <- rownames(theta)
    return(list(basis = basis))
}

#' Select Informative Genes used in the deconvolution
#'
#' @param basis Reference basis matrix.
#' @param sc_eset scRNAseq data along with meta data stored in the S4 class
#' format (SingleCellExperiment).
#' @param commongene common genes between scRNAseq count data and spatial
#' resolved transcriptomics data.
#' @param ct_select vector of cell type names that you are interested in to
#' deconvolute, default as NULL. If NULL, then use all cell types provided by
#' single cell dataset;
#' @param ct_varname character, the name of the column in metaData that
#' specifies the cell type annotation information
#'
#' @importFrom SummarizedExperiment assays colData
#' @importFrom stats quantile var
#' @return a vector of informative genes selected
#'

select_info <- function(basis, sc_eset, commongene, ct_select, ct_varname) {
    ##### log2 mean fold change >0.5
    gene1 <- lapply(ct_select, function(ict) {
        rest <- rowMeans(basis[, colnames(basis) != ict])
        fc <- log((basis[, ict] + 1e-06)) - log((rest + 1e-06))
        rownames(basis)[fc > 1.25 & basis[, ict] > 0]
    })
    gene1 <- unique(unlist(gene1))
    gene_int <- intersect(gene1, commongene)
    counts <- assays(sc_eset)$counts
    counts <- counts[rownames(counts) %in% gene_int, ]
    
    ##### only check the cell type that contains at least 2 cells
    ct_select <- names(table(colData(sc_eset)[, ct_varname])
        )[table(colData(sc_eset)[, ct_varname]) > 1]
    sd_within <- vapply(ct_select, function(ict) {
        temp <- counts[, colData(sc_eset)[, ct_varname] == ict]
        apply(temp, 1, stats::var) / apply(temp, 1, mean)
    }, FUN.VALUE = numeric(nrow(counts)))
    
    ##### remove the outliers that have high dispersion across cell types
    gene2 <- rownames(sd_within)[apply(sd_within,1,mean,na.rm = TRUE) < 
                                quantile(
                                    apply(sd_within,1,mean,na.rm = TRUE),
                                    prob = 0.99,
                                    na.rm = TRUE)]
    return(gene2)
}


#' Spatially Informed Cell Type Deconvolution for Spatial Transcriptomics
#' by CARD
#'
#' @param sc_count Raw scRNA-seq count data, each column is a cell and each row
#' is a gene.
#' @param sc_meta data frame, with each row representing the cell type and/or
#' sample information of a specific cell. The row names of this data frame
#' should match exactly with the column names of the sc_count data
#' @param spatial_count Raw spatial resolved transcriptomics data, each column
#' is a spatial location, and each row is a gene.
#' @param spatial_location data frame, with two columns representing the x and
#' y coordinates of the spatial location. The rownames of this data frame
#' should match eaxctly with the columns of the spatial_count.
#' @param sce a \code{SingleCellExperiment} object containing scRNA-seq count 
#' data in the \code{counts} assay, and cell types and sample information in 
#' the colData.
#' @param spe a \code{SpatialExperiment} object containing spatial data in 
#' the \code{counts} assay, and spatial coordinates in the spatialCoords.
#' @param ct_varname character, the name of the column in metaData that
#' specifies the cell type annotation information
#' @param ct_select vector of cell type names that you are interested in to
#' deconvolute, default as NULL. If NULL, then use all cell types provided by
#' single cell dataset;
#' @param sample_varname character,the name of the column in metaData that
#' specifies the sample information. If NULL, we just use the whole as one
#' sample.
#' @param mincountgene Minimum counts for each gene
#' @param mincountspot Minimum counts for each spatial location
#' @importFrom Rcpp sourceCpp
#' @importFrom MCMCpack rdirichlet
#' @importFrom fields rdist
#' @importFrom S4Vectors metadata
#' @importFrom S4Vectors metadata<-
#' @return Returns a SpatialExperiment object with estimated cell type 
#' proportion stored in object$Proportion_CARD.
#'
#' @export
#' @examples
#' library(RcppML)
#' library(NMF)
#' library(RcppArmadillo)
#' data(spatial_count)
#' data(spatial_location)
#' data(sc_count)
#' data(sc_meta)
#' CARD_obj <- CARD_deconvolution(
#'     sc_count = sc_count,
#'     sc_meta = sc_meta,
#'     spatial_count = spatial_count,
#'     spatial_location = spatial_location,
#'     ct_varname = "cellType",
#'     ct_select = unique(sc_meta$cellType),
#'     sample_varname = "sampleInfo",
#'     mincountgene = 100,
#'     mincountspot = 5
#' )
CARD_deconvolution <- function(
        sc_count, 
        sc_meta, 
        spatial_count, 
        spatial_location,
        ct_varname, 
        ct_select, 
        sample_varname,
        mincountgene = 100, 
        mincountspot = 5,
        sce = NULL, 
        spe = NULL) {
    ##### Check
    if(is.null(sc_count) ||
        is.null(sc_meta) || 
        is.null(spatial_count) || 
        is.null(spatial_location)){
        if(is.null(sce) || is.null(spe)){
            stop("Please provide SingleCellExperiment object and 
            SpatialExperiment object")
        }
    }else{
        if (!is.vector(sc_count) && 
            !is.matrix(sc_count) && 
            !inherits(sc_count, "sparseMatrix")) {
            stop("scRNASeq counts has to be of following forms: vector,
        matrix or sparseMatrix")
        }
        if (!is.vector(spatial_count) && 
            !is.matrix(spatial_count) && 
            !inherits(spatial_count, "sparseMatrix")) {
            stop("SRT counts has to be of following forms: vector,
        matrix or sparseMatrix")
        }
    }
    if (is.null(ct_varname)) {
        stop("Please provide the column name indicating the cell type 
        information in the meta data of scRNA-seq")
    }
    if(!is.null(sample_varname) && 
        !is.character(sample_varname) || 
        length(sample_varname) > 1){
        stop("Please input valid sample variable name")
    }
    if(!is.null(ct_select) && 
        !is.character(ct_select) ){
        stop("Please input valid cell types")
    }
    
    ##### create CARD object
    CARD_object <- createCARDObject(
        sc_count = sc_count,
        sc_meta = sc_meta,
        sce = sce,
        spe = spe,
        spatial_count = spatial_count,
        spatial_location = spatial_location,
        ct_varname = ct_varname,
        ct_select = ct_select,
        sample_varname = sample_varname,
        mincountgene = mincountgene,
        mincountspot = mincountspot) 
    
    ##### run deconvolution
    ct_select <- CARD_object@info_parameters$ct.select
    ct_varname <- CARD_object@info_parameters$ct.varname
    sample_varname <- CARD_object@info_parameters$sample.varname
    message("## create reference matrix from scRNASeq...\n")
    sc_eset <- CARD_object@sc_eset
    basis_ref <- create_ref(sc_eset, ct_select, ct_varname, sample_varname)
    basis <- basis_ref$basis
    basis <- basis[, colnames(basis) %in% ct_select]
    basis <- basis[, match(ct_select, colnames(basis))]
    spatial_count <- CARD_object@spatial_countMat
    commongene <- intersect(rownames(spatial_count), rownames(basis))
    
    ##### remove mitochondrial and ribosomal genes
    commongene <- commongene[!(commongene %in%
                    commongene[grep("mt-", commongene)])]
    message("## Select Informative Genes! ...\n")
    common <- select_info(basis, sc_eset, commongene, ct_select, ct_varname)
    xinput <- spatial_count
    rm(spatial_count)
    B <- basis
    rm(basis)
    
    ##### match the common gene names
    xinput <- xinput[order(rownames(xinput)), ]
    B <- B[order(rownames(B)), ]
    B <- B[rownames(B) %in% common, ]
    xinput <- xinput[rownames(xinput) %in% common, ]
    
    ##### filter out non expressed genes or cells again
    xinput <- xinput[rowSums(xinput) > 0, ]
    xinput <- xinput[, colSums(xinput) > 0]
    
    ##### normalize count data
    colsumvec <- colSums(xinput)
    xinput_norm <- sweep(xinput, 2, colsumvec, "/")
    B <- B[rownames(B) %in% rownames(xinput_norm), ]
    B <- B[match(rownames(xinput_norm), rownames(B)), ]
    
    ##### spatial location
    spatial_location <- CARD_object@spatial_location
    spatial_location <- spatial_location[rownames(spatial_location) %in%
                                    colnames(xinput_norm), ]
    spatial_location <- spatial_location[match(
        colnames(xinput_norm),
        rownames(spatial_location)
    ), ]
    
    #### normalize the coordinates without changing the shape and relative
    ##### position
    norm_cords <- spatial_location[, c("x", "y")]
    norm_cords$x <- norm_cords$x - min(norm_cords$x)
    norm_cords$y <- norm_cords$y - min(norm_cords$y)
    scalefactor <- max(norm_cords$x, norm_cords$y)
    norm_cords$x <- norm_cords$x / scalefactor
    norm_cords$y <- norm_cords$y / scalefactor
    
    ##### initialize the proportion matrix
    ed <- rdist(as.matrix(norm_cords)) ## Euclidean distance matrix
    message("## Deconvolution Starts! ...\n")
    vint1 <- as.matrix(rdirichlet(ncol(xinput_norm), rep(10, ncol(B))))
    colnames(vint1) <- colnames(B)
    rownames(vint1) <- colnames(xinput_norm)
    b <- rep(0, length(ct_select))
    
    ##### parameters that need to be set
    isigma <- 0.1
    epsilon <- 1e-04 #### convergence epsion
    phi <- c(0.01, 0.1, 0.3, 0.5, 0.7, 0.9, 0.99) #### grided values for phi
    kernel_mat <- exp(-ed^2 / (2 * isigma^2))
    diag(kernel_mat) <- 0
    rm(ed)
    rm(xinput)
    rm(norm_cords)
    gc()
    
    ##### scale the xinput_norm and B to speed up the convergence.
    mean_X <- mean(xinput_norm)
    mean_B <- mean(B)
    xinput_norm <- xinput_norm * 0.1 / mean_X
    B <- B * 0.1 / mean_B
    gc()
    reslist <- list()
    obj <- c()
    for (iphi in seq_len(length(phi))) {
        res <- CARDref(
            XinputIn = as.matrix(xinput_norm), 
            UIn = as.matrix(B),
            WIn = kernel_mat,
            phiIn = phi[iphi],
            max_iterIn = 1000,
            epsilonIn = epsilon,
            initV = vint1,
            initb = rep(0, ncol(B)),
            initSigma_e2 = 0.1,
            initLambda = rep(10, length(ct_select))
        )
        rownames(res$V) <- colnames(xinput_norm)
        colnames(res$V) <- colnames(B)
        reslist[[iphi]] <- res
        obj <- c(obj, res$Obj)
    }
    optimal_ind <- which(obj == max(obj))
    optimal <- optimal_ind[length(optimal_ind)]
    optimalphi <- phi[optimal]
    optimalres <- reslist[[optimal]]
    message("## Deconvolution Finish! ...\n")
    CARD_object@info_parameters$phi <- optimalphi
    CARD_object@Proportion_CARD <- sweep(
        optimalres$V, 1, rowSums(optimalres$V),
        "/"
    )
    CARD_object@algorithm_matrix <- list(
        B = B * mean_B / 0.1,
        Xinput_norm = xinput_norm * mean_X / 0.1, 
        Res = optimalres
    )
    CARD_object@spatial_location <- spatial_location
    
    ##### return SpatialExperiment Object
    spe_out <- SpatialExperiment(
        assay = list(spatial_countMat = CARD_object@spatial_countMat), 
        spatialCoords = as.matrix(CARD_object@spatial_location)
    )
    colData(spe_out)$Proportion_CARD <- CARD_object@Proportion_CARD
    
    ##### remove sample_id 
    spe_out@colData@listData[["sample_id"]] <- NULL
    metadata(spe_out)$project <- CARD_object@project
    metadata(spe_out)$info_parameters <- CARD_object@info_parameters
    metadata(spe_out)$algorithm_matrix <- CARD_object@algorithm_matrix
    metadata(spe_out)$sc_eset <- CARD_object@sc_eset
    return(spe_out)
}

.onLoad <- function(libname, pkgname) {
    # CRAN Note avoidance
    utils::globalVariables(
        # variable names for ggplot
        c(
            "x",
            "y",
            "value"
        )
    )
    # verify_NA_ALIAS(0L)
    invisible(NULL)
}

#' Extension of CARD into a reference-free version of
#' deconvolution: CARDfree.
#'
#' @param markerlist a list of marker genes, with each element of the list
#' being the vector of cell type specific marker genes
#' @param spatial_count Raw spatial resolved transcriptomics data, each column
#' is a spatial location, and each row is a gene.
#' @param spatial_location data frame, with two columns representing the x and
#' y coordinates of the spatial location. The rownames of this data frame
#' should match eaxctly with the columns of the spatial_count.
#' @param spe a \code{SpatialExperiment} object containing spatial data in 
#' the \code{counts} assay, and spatial coordinates in the spatialCoords.
#' @param mincountgene Minimum counts for each gene
#' @param mincountspot Minimum counts for each spatial location
#' @importFrom Rcpp sourceCpp
#' @importFrom fields rdist
#' @importFrom S4Vectors metadata
#' @importFrom S4Vectors metadata<-
#' @return Returns a SpatialExperiment object with estimated cell type 
#' proportion stored in object$Proportion_CARD. Because this is a 
#' reference-free version, the columns of estimated proportion is not cell 
#' type but cell type cluster
#'
#' @export
#' @examples
#' library(RcppML)
#' library(NMF)
#' library(RcppArmadillo)
#' data(markerList)
#' data(spatial_count)
#' data(spatial_location)
#' CARDfree_obj <- CARD_refFree(
#' markerlist = markerList[8:16],
#' spatial_count = spatial_count[1:2500, ],
#' spatial_location = spatial_location,
#' mincountgene = 100,
#' mincountspot = 5
#' )
#'
CARD_refFree <- function(
        markerlist, 
        spatial_count, 
        spatial_location,
        mincountgene = 100, 
        mincountspot = 5,
        spe = NULL) {
    ##### Check
    if(is.null(spatial_count) || is.null(spatial_location)){
        if(is.null(spe)){
            stop("Please provide SpatialExperiment object")}
    }else{
        if (!is.vector(spatial_count) && 
            !is.matrix(spatial_count) && 
            !inherits(spatial_count, "sparseMatrix")) {
            stop("SRT counts has to be of following forms: vector,
        matrix or sparseMatrix")
        }
    }
    if(is.null(markerlist)){
        stop("Please provide the marker list for CARDfree")
    }
    
    ##### Create CARD refFree obejct
    CARDfree_object <- createCARDfreeObject(
        markerlist = markerlist,
        spatial_count = spatial_count,
        spatial_location = spatial_location,
        mincountgene = mincountgene,
        mincountspot = mincountspot,
        spe = spe
    )
    
    #### Run CARD refFree deconvolution
    ##### load in spatial transcriptomics data stored in CARDfree_object
    spatial_countmat <- CARDfree_object@spatial_countMat
    spatial_location <- CARDfree_object@spatial_location
    
    ##### load in markerlist number of cell type clusters
    markerlist <- CARDfree_object@markerList
    numK <- length(markerlist)
    marker <- unique(unlist(markerlist))
    message(
        "## Number of unique marker genes: ", 
        length(marker), 
        " for ",
        numK, 
        " cell types ...\n"
    )
    commongene <- intersect(
        toupper(rownames(spatial_countmat)),
        toupper(marker)
    )
    
    ##### remove mitochondrial and ribosomal genes
    commongene <- commongene[!(commongene %in% 
                            commongene[grep("mt-",commongene)])]
    if (length(commongene) < numK * 10) {
        stop("## STOP! The average number of unique marker genes for each cell
            type is less than 20 ...\n")
    }
    xinput <- spatial_countmat[order(rownames(spatial_countmat)), ]
    xinput <- xinput[order(rownames(xinput)), ]
    xinput <- xinput[toupper(rownames(xinput)) %in% commongene, ]
    xinput <- xinput[rowSums(xinput) > 0, ]
    xinput <- xinput[, colSums(xinput) > 0]
    xinput_norm <- sweep(xinput, 2, colSums(xinput), "/")
    
    ##### initialization
    if (ncol(xinput_norm) < 5000) {
        if (.Platform$OS.type == "windows") {
            sink("NUL")
        } else {
            sink("/dev/null")
        }
        nmfout <- invisible(NMF::nmf(as.matrix(xinput_norm), numK))
        sink()
        B <- nmfout@fit@W
        vint1 <- as.matrix(t(nmfout@fit@H))
        rownames(vint1) <- colnames(xinput_norm)
    } else {
        if (.Platform$OS.type == "windows") {
            sink("NUL")
        } else {
            sink("/dev/null")
        }
        nmfout <- invisible(RcppML::nmf(as.matrix(xinput_norm), numK))
        sink()
        B <- nmfout$w
        rownames(B) <- rownames(xinput_norm)
        vint1 <- as.matrix(t(nmfout$h))
        rownames(vint1) <- colnames(xinput_norm)
    }
    spatial_location <- spatial_location[rownames(spatial_location) %in%
                                        colnames(xinput_norm), ]
    spatial_location <- spatial_location[
        match(colnames(xinput_norm),rownames(spatial_location)), ]
    norm_cords <- spatial_location[, c("x", "y")]
    norm_cords$x <- norm_cords$x - min(norm_cords$x)
    norm_cords$y <- norm_cords$y - min(norm_cords$y)
    scalefactor <- max(norm_cords$x, norm_cords$y)
    norm_cords$x <- norm_cords$x / scalefactor
    norm_cords$y <- norm_cords$y / scalefactor
    
    ##### Euclidiean distance
    ed <- rdist(as.matrix(norm_cords)) ## Euclidean distance matrix
    b <- rep(0, ncol(B))
    
    ##### parameters that need to be set
    isigma <- 0.1
    epsilon <- 1e-04 #### convergence epsion
    phi <- c(0.01, 0.1, 0.3, 0.5, 0.7, 0.9, 0.99) #### grided values for phi
    kernel_mat <- exp(-ed^2 / (2 * isigma^2))
    diag(kernel_mat) <- 0
    rm(ed)
    rm(xinput)
    rm(norm_cords)
    gc()
    
    ##### scale the xinput_norm and B to speed up the convergence.
    mean_X <- mean(xinput_norm)
    mean_B <- mean(B)
    xinput_norm <- xinput_norm * 0.1 / mean_X
    B <- B * 0.1 / mean_B
    gc()
    reslist <- list()
    obj <- c()
    for (iphi in seq_len(length(phi))) {
        res <- CARDfree(
            XinputIn = as.matrix(xinput_norm),
            UIn = as.matrix(B),
            WIn = kernel_mat,
            phiIn = phi[iphi],
            max_iterIn = 1000,
            epsilonIn = epsilon,
            initV = vint1,
            initb = rep(0, ncol(B)),
            initSigma_e2 = 0.1,
            initLambda = rep(10, ncol(B))
        )
        rownames(res$V) <- colnames(xinput_norm)
        colnames(res$V) <- paste0("CT", seq_len(ncol(B)))
        reslist[[iphi]] <- res
        obj <- c(obj, res$Obj)
    }
    optimal_ind <- which(obj == max(obj))
    optimal <- optimal_ind[length(optimal_ind)]
    optimalphi <- phi[optimal]
    optimalres <- reslist[[optimal]]
    message("## Deconvolution Finish! ...\n")
    CARDfree_object@info_parameters$phi <- optimalphi
    CARDfree_object@Proportion_CARD <- sweep(
        optimalres$V, 
        1,
        rowSums(optimalres$V),
        "/"
    )
    CARDfree_object@algorithm_matrix <- list(
        B = optimalres$B * mean_B / 0.1,
        Xinput_norm = 
            xinput_norm * mean_X / 0.1, 
        Res = optimalres
    )
    CARDfree_object@spatial_location <- spatial_location
    CARDfree_object@estimated_refMatrix <- optimalres$B * mean_B / 0.1
    rownames(CARDfree_object@estimated_refMatrix) <- rownames(B)
    colnames(CARDfree_object@estimated_refMatrix) <- colnames(B)
    
    ##### store as a SpatialExperiemnt object
    spe_out <- SpatialExperiment(
        assay = list(spatial_countMat = CARDfree_object@spatial_countMat), 
        spatialCoords = as.matrix(CARDfree_object@spatial_location))
    ## remove sample_id 
    spe_out@colData@listData[["sample_id"]] <- NULL
    colData(spe_out)$Proportion_CARD <- CARDfree_object@Proportion_CARD
    metadata(spe_out)$project <- CARDfree_object@project
    metadata(spe_out)$info_parameters <- CARDfree_object@info_parameters
    metadata(spe_out)$algorithm_matrix <- CARDfree_object@algorithm_matrix
    metadata(spe_out)$markerList <- CARDfree_object@markerList
    metadata(spe_out)$estimated_refMatrix <- 
        CARDfree_object@estimated_refMatrix
    return(spe_out)
}

test_that("Reference-Based Deconvolution works well", {
    expect_true(exists("CARD_obj"))
    expect_true("Proportion_CARD" %in% names(colData(CARD_obj)))
})


test_that("Reference-free Deconvolution works well", {
    library(NMF)
    data(spatial_count)
    data(spatial_location)
    data(markerList)
    markerList2 <- markerList[6:10]
    CARDfree_obj = CARD_refFree(
        markerlist = markerList2,
        spatial_count = spatial_count,
        spatial_location = spatial_location,
        mincountgene = 10,
        mincountspot =5
    ) 
    res <- rowSums(CARDfree_obj$Proportion_CARD)
    expect_equal(as.numeric(res), as.numeric(rep(1, 428)))
})

test_that("Refined spatial map works well", {
    ## use global reference-based deconvolution result
    expect_no_error({
        CARD_obj <- CARD_imputation(CARD_obj, num_grids = 2000, ineibor = 10, 
                                    exclude = NULL)
    })
})

test_that("Single cell resolution mapping works well", {
    ## use global reference-based deconvolution result
    expect_no_error({
        scMapping <- CARD_scmapping(CARD_obj,shapeSpot="Square",
                                    numcell=20,ncore=1)
    })
})

#' marker gene list
#'
#' The marker gene list is a list format with each element of the list being the
#' cell type specific gene markers.
#'
#'
#' @docType data
#' @keywords datasets
#' @usage data(markerList)
#' @name markerList
"markerList"

#' Visualize the spatial distribution of cell type proportion
#'
#' @param proportion Data frame, cell type proportion estimated by CARD in
#' either original resolution or enhanced resolution.
#' @param spatial_location Data frame, spatial location information.
#' @param ct_visualize Vector of selected cell type names that are
#' interested to visualize
#' @param colors Vector of color names that you want to use, if NULL, we will
#' use the default color scale c("lightblue","lightyellow","red")
#' @param NumCols Numeric, number of columns in the figure panel, it depends on
#' the number of cell types you want to visualize.
#' @param pointSize Size of each point used for plotting
#' @import ggplot2
#' @importFrom reshape2 melt
#' @return Returns a ggplot2 figure.
#'
#' @export
#' @examples
#' library(ggplot2)
#' library(SpatialExperiment)
#' data(spatial_count)
#' data(spatial_location)
#' data(sc_count)
#' data(sc_meta)
#' CARD_obj <- CARD_deconvolution(
#'     sc_count = sc_count,
#'     sc_meta = sc_meta,
#'     spatial_count = spatial_count,
#'     spatial_location = spatial_location,
#'     ct_varname = "cellType",
#'     ct_select = unique(sc_meta$cellType),
#'     sample_varname = "sampleInfo",
#'     mincountgene = 100,
#'     mincountspot = 5
#' )
#' ct_visualize <- c(
#'     "Acinar_cells", "Cancer_clone_A", "Cancer_clone_B",
#'     "Ductal_terminal_ductal_like", "Ductal_CRISP3_high-centroacinar_like",
#'     "Ductal_MHC_Class_II", "Ductal_APOL1_high-hypoxic", "Fibroblasts"
#' )
#' CARD_visualize_prop(
#'     proportion = CARD_obj$Proportion_CARD,
#'     spatial_location = spatialCoords(CARD_obj),
#'     ct_visualize = ct_visualize,
#'     colors = c("lightblue", "lightyellow", "red"),
#'     NumCols = 4,
#'     pointSize = 3.0
#' )
#'
CARD_visualize_prop <- function(
        proportion, 
        spatial_location,
        ct_visualize = ct_visualize,
        colors = c("lightblue", "lightyellow", "red"),
        NumCols, 
        pointSize = 3.0) {
    #####Preapare dataframe for plotting
    if (is.null(colors)) {
        colors <- c("lightblue", "lightyellow", "red")
    } else {
        colors <- colors
    }
    res_CARD <- as.data.frame(proportion)
    res_CARD <- res_CARD[, order(colnames(res_CARD))]
    location <- as.data.frame(spatial_location)
    if (sum(rownames(res_CARD) == rownames(location)) != nrow(res_CARD)) {
        stop("The rownames of proportion data does not match with the
            rownames of spatial location data")
    }
    ct_select <- ct_visualize
    res_CARD <- res_CARD[, ct_select]
    if (!is.null(ncol(res_CARD))) {
        res_CARD_scale <- as.data.frame(apply(res_CARD, 2, function(x) {
            (x - min(x)) / (max(x) - min(x))}))
    } else {
        res_CARD_scale <-
            as.data.frame((res_CARD - min(res_CARD)) /
                        (max(res_CARD) - min(res_CARD)))
        colnames(res_CARD_scale) <- ct_visualize
    }
    res_CARD_scale$x <- as.numeric(location$x)
    res_CARD_scale$y <- as.numeric(location$y)
    mdata <- melt(res_CARD_scale, id.vars = c("x", "y"))
    colnames(mdata)[3] <- "Cell_Type"
    b <- c(0, 1)
    
    #####Plot
    p <- ggplot(mdata, aes(x, y)) +
        geom_point(aes(colour = value), size = pointSize) +
        scale_color_gradientn(colours = colors) +
        scale_x_discrete(expand = c(0, 1)) +
        scale_y_discrete(expand = c(0, 1)) +
        facet_wrap(~Cell_Type, ncol = NumCols) +
        coord_fixed() +
        theme(
            plot.margin = margin(0.1, 0.1, 0.1, 0.1, "cm"),
            panel.background = element_blank(),
            plot.background = element_blank(),
            panel.border = element_rect(colour = "grey89", fill = NA, 
                                        linewidth = 0.5),
            axis.text = element_blank(),
            axis.ticks = element_blank(),
            axis.title = element_blank(),
            legend.title = element_text(size = 14, face = "bold"),
            legend.text = element_text(size = 11),
            strip.text = element_text(size = 12, face = "bold"),
            legend.key = element_rect(colour = "transparent", fill = "white"),
            legend.key.size = unit(0.45, "cm")
        )
    return(p)
}

#' Visualize the spatial distribution of two cell type proportions on
#' the same plot
#'
#' @param proportion Data frame, cell type proportion estimated by CARD in
#' either original resolution or enhanced resolution.
#' @param spatial_location Data frame, spatial location information.
#' @param ct2_visualize Vector of selected two cell type names that are
#' interested to visualize, here we only focus on two cell types
#' @param colors list of color names that you want to use for each cell type,
#' if NULL, we will use the default color scale list
#' list(c("lightblue","lightyellow","red"),c("lightblue","lightyellow","black")
#'
#' @import ggplot2
#' @importFrom reshape2 melt
#' @return Returns a ggplot2 figure.
#'
#' @export
#' @examples
#' library(ggplot2)
#' library(SpatialExperiment)
#' data(spatial_count)
#' data(spatial_location)
#' data(sc_count)
#' data(sc_meta)
#' CARD_obj <- CARD_deconvolution(
#'     sc_count = sc_count,
#'     sc_meta = sc_meta,
#'     spatial_count = spatial_count,
#'     spatial_location = spatial_location,
#'     ct_varname = "cellType",
#'     ct_select = unique(sc_meta$cellType),
#'     sample_varname = "sampleInfo",
#'     mincountgene = 100,
#'     mincountspot = 5
#' )
#' CARD_visualize_prop_2CT(
#'     proportion = CARD_obj$Proportion_CARD,
#'     spatial_location = spatialCoords(CARD_obj),
#'     ct2_visualize = c("Cancer_clone_A", "Cancer_clone_B"),
#'     colors = list(c("lightblue", "lightyellow", "red"), c(
#'         "lightblue", "lightyellow",
#'         "black"
#'     ))
#' )
#'
CARD_visualize_prop_2CT <- function(
        proportion, 
        spatial_location,
        ct2_visualize = ct2_visualize,
        colors = NULL) {
    #####Preapare dataframe for plotting
    if (is.null(colors)) {
        colors <- list(
            c("lightblue", "lightyellow", "red"),
            c("lightblue", "lightyellow", "black")
        )
    } else {
        colors <- colors
    }
    res_CARD <- as.data.frame(proportion)
    res_CARD <- res_CARD[, order(colnames(res_CARD))]
    location <- as.data.frame(spatial_location)
    if (sum(rownames(res_CARD) == rownames(location)) != nrow(res_CARD)) {
        stop("The rownames of proportion data does not match with the
            rownames of spatial location data")
    }
    ct_select <- ct2_visualize
    res_CARD <- res_CARD[, ct_select]
    res_CARD_scale <- as.data.frame(apply(res_CARD, 2, function(x) {
        (x - min(x)) / (max(x) - min(x))
    }))
    res_CARD_scale$x <- as.numeric(location$x)
    res_CARD_scale$y <- as.numeric(location$y)
    mdata <- melt(res_CARD_scale, id.vars = c("x", "y"))
    colnames(mdata)[3] <- "Cell_Type"
    b <- c(0, 1)
    
    #####Plot
    p <- ggplot() +
        geom_point(
            data = mdata[mdata$Cell_Type == ct2_visualize[1], ],
            aes(x = x,y = y,color = value), 
            shape = 21, 
            size = 5
        ) +
        scale_color_gradientn(colours = colors[[1]]) +
        geom_point(
            data = mdata[mdata$Cell_Type == ct2_visualize[2], ],
            aes(x = x,y = y,fill = value), 
            color = "white", 
            shape = 22, 
            size = 2
        ) +
        scale_fill_gradientn(colours = colors[[2]]) +
        scale_x_discrete(expand = c(0, 1)) +
        scale_y_discrete(expand = c(0, 1)) +
        coord_fixed() +
        theme(
            plot.margin = margin(0.1, 0.1, 0.1, 0.1, "cm"),
            panel.background = element_blank(),
            plot.background = element_blank(),
            panel.border = element_rect(
                colour = "grey89", 
                fill = NA, 
                linewidth = 0.5),
            axis.text = element_blank(),
            axis.ticks = element_blank(),
            axis.title = element_blank(),
            legend.title = element_text(size = 14, face = "bold"),
            legend.text = element_text(size = 11),
            strip.text = element_text(size = 12, face = "bold"),
            legend.key.size = unit(0.45, "cm")
        ) +
        labs(colour = ct2_visualize[1], fill = ct2_visualize[2])
    return(p)
}

#' Visualize the spatial distribution of cell type proportion in a geom
#' scatterpie plot
#'
#' @param proportion Data frame, cell type proportion estimated by CARD in
#' either original resolution or enhanced resolution.
#' @param spatial_location Data frame, spatial location information.
#' @param colors Vector of color names that you want to use, if NULL, we will
#' use the color palette "Spectral" from RColorBrewer package.
#' @param radius Numeric value about the radius of each pie chart, if NULL, we
#' will calculate it inside the function.
#'
#' @import ggplot2
#' @importFrom RColorBrewer brewer.pal
#' @importFrom scatterpie geom_scatterpie
#' @importFrom grDevices colorRampPalette
#' @importFrom gtools mixedsort
#' @return Returns a ggplot2 figure.
#'
#' @export
#' @examples
#' library(ggplot2)
#' library(SpatialExperiment)
#' data(spatial_count)
#' data(spatial_location)
#' data(sc_count)
#' data(sc_meta)
#' CARD_obj <- CARD_deconvolution(
#'     sc_count = sc_count,
#'     sc_meta = sc_meta,
#'     spatial_count = spatial_count,
#'     spatial_location = spatial_location,
#'     ct_varname = "cellType",
#'     ct_select = unique(sc_meta$cellType),
#'     sample_varname = "sampleInfo",
#'     mincountgene = 100,
#'     mincountspot = 5
#' )
#' colors <- c(
#'     "#FFD92F", "#4DAF4A", "#FCCDE5", "#D9D9D9", "#377EB8", "#7FC97F",
#'     "#BEAED4", "#FDC086", "#FFFF99", "#386CB0", "#F0027F", "#BF5B17", 
#'     "#666666", "#1B9E77", "#D95F02", "#7570B3", "#E7298A", "#66A61E", 
#'     "#E6AB02", "#A6761D"
#' )
#' CARD_visualize_pie(
#'     proportion = CARD_obj$Proportion_CARD,
#'     spatial_location = spatialCoords(CARD_obj),
#'     colors = colors,
#'     radius = 0.52
#' )
CARD_visualize_pie <- function(
        proportion, 
        spatial_location, 
        colors = NULL,
        radius = NULL) {
    #####Preapare dataframe for plotting
    res_CARD <- as.data.frame(proportion)
    res_CARD <- res_CARD[, mixedsort(colnames(res_CARD))]
    location <- as.data.frame(spatial_location)
    if (sum(rownames(res_CARD) == rownames(location)) != nrow(res_CARD)) {
        stop("The rownames of proportion data does not match with the rownames
        of spatial location data")
    }
    colorCandidate <- c(
        "#1e77b4", "#ff7d0b", "#ceaaa3", "#2c9f2c", "#babc22",
        "#d52828", "#9267bc",
        "#8b544c", "#e277c1", "#d42728", "#adc6e8", "#97df89", "#fe9795", 
        "#4381bd",
        "#f2941f", "#5aa43a", "#cc4d2e", "#9f83c8", "#91675a",
        "#da8ec8", "#929292", "#c3c237", "#b4e0ea", "#bacceb", "#f7c685",
        "#dcf0d0", "#f4a99f", "#c8bad8",
        "#F56867", "#FEB915", "#C798EE", "#59BE86", "#7495D3",
        "#D1D1D1", "#6D1A9C", "#15821E", "#3A84E6", "#997273",
        "#787878", "#DB4C6C", "#9E7A7A", "#554236", "#AF5F3C",
        "#93796C", "#F9BD3F", "#DAB370", "#877F6C", "#268785",
        "#f4f1de", "#e07a5f", "#3d405b", "#81b29a", "#f2cc8f", "#a8dadc",
        "#f1faee", "#f08080"
    )
    if (is.null(colors)) {
        if (ncol(res_CARD) > length(colorCandidate)) {
            colors <- colorRampPalette(colorCandidate)(ncol(res_CARD))
        } else {
            colors <-
                colorCandidate[sample(
                    seq_len(length(colorCandidate)),
                    ncol(res_CARD)
                )]
        }
    } else {
        colors <- colors
    }
    data <- cbind(res_CARD, location)
    ct_select <- colnames(res_CARD)
    if (is.null(radius)) {
        radius <- (max(data$x) - min(data$x)) * (max(data$y) - min(data$y))
        radius <- radius / nrow(data)
        radius <- radius / pi
        radius <- sqrt(radius) * 0.85
    } else {
        #### avoid the situation when the radius does not generate the correct
        #### figure
        radius <- radius
    }
    
    #####Plot
    p <- 
        ggplot() +
            geom_scatterpie(
                data = data,
                aes(x = x, y = y, r = radius),
                cols = ct_select,
                color = NA
            ) +
            coord_fixed(ratio = 1 * max(data$x) / max(data$y)) +
            scale_fill_manual(values = colors) +
            theme(
                plot.margin = margin(0.1, 0.1, 0.1, 0.1, "cm"),
                panel.background = element_blank(),
                plot.background = element_blank(),
                panel.border = element_rect(
                    colour = "grey89", 
                    fill = NA, 
                    linewidth = 0.5),
                axis.text = element_blank(),
                axis.ticks = element_blank(),
                axis.title = element_blank(),
                legend.title = element_text(size = 16, face = "bold"),
                legend.text = element_text(size = 15),
                legend.key = element_rect(
                    colour = "transparent", 
                    fill = "white"),
                legend.key.size = unit(0.45, "cm"),
                strip.text = element_text(size = 16, face = "bold"),
                legend.position = "bottom"
            ) +
            guides(fill = guide_legend(title = "Cell Type"))
    
    return(p)
}

#' Visualize the cell type proportion correlation
#'
#' @param proportion Data frame, cell type proportion estimated by CARD in
#' either original resolution or enhanced resolution.
#' @param colors Vector of color names that you want to use, if NULL, we will
#' use the default color scale c("#91a28c","white","#8f2c37")
#'
#' @import ggcorrplot
#' @importFrom stats cor
#' @return Returns a ggcorrplot figure.
#'
#' @export
#' @examples
#' library(ggplot2)
#' data(spatial_count)
#' data(spatial_location)
#' data(sc_count)
#' data(sc_meta)
#' CARD_obj <- CARD_deconvolution(
#'     sc_count = sc_count,
#'     sc_meta = sc_meta,
#'     spatial_count = spatial_count,
#'     spatial_location = spatial_location,
#'     ct_varname = "cellType",
#'     ct_select = unique(sc_meta$cellType),
#'     sample_varname = "sampleInfo",
#'     mincountgene = 100,
#'     mincountspot = 5
#' )
#' CARD_visualize_Cor(CARD_obj$Proportion_CARD, colors = NULL)
CARD_visualize_Cor <- function(proportion, colors = colors) {
    #####Preapare dataframe for plotting
    proportion <- proportion[, order(colnames(proportion))]
    cor_CARD <- cor(as.matrix(proportion))
    if (is.null(colors)) {
        colors <- c("#91a28c", "white", "#8f2c37")
    } else {
        colors <- colors
    }
    
    #####Plot
    p <- ggcorrplot(
        cor_CARD,
        hc.order = FALSE,
        outline.color = "white",
        tl.srt = 60,
        tl.cex = 18,
        lab_size = 7,
        colors = colors
    ) +
        theme(
            plot.margin = margin(0.1, 0.1, 0.1, 0.1, "cm"),
            panel.background = element_blank(),
            plot.background = element_blank(),
            panel.border = element_rect(
                colour = "grey89", 
                fill = NA, 
                linewidth = 0.5),
            axis.text = element_text(size = 12),
            axis.ticks = element_blank(),
            axis.title = element_blank(),
            legend.title = element_text(size = 16, face = "bold"),
            legend.text = element_text(size = 16),
            legend.key = element_rect(
                colour = "transparent", 
                fill = "white"),
            legend.key.size = unit(0.45, "cm")
        ) +
        coord_fixed() +
        ggtitle("Correlation") +
        theme(plot.title = element_text(size = 22, face = "bold"))
    return(p)
}

#' Visualize the spatial distribution of cell type proportion
#'
#' @param spatial_expression Data frame, spatial gene expression in either
#' original resolution or enhanced resolution.
#' @param spatial_location Data frame, spatial location information.
#' @param gene_visualize Vector of selected gene names that are interested to
#' visualize
#' @param colors Vector of color names that you want to use, if NULL, we will
#' use the default color scale in virdis palette
#' @param NumCols Numeric, number of columns in the figure panel, it depends on
#' the number of cell types you want to visualize.
#'
#' @import ggplot2
#' @return Returns a ggplot2 figure.
#'
#' @export
#' @examples
#' library(ggplot2)
#' library(SummarizedExperiment)
#' library(SpatialExperiment)
#' data(spatial_count)
#' data(spatial_location)
#' data(sc_count)
#' data(sc_meta)
#' CARD_obj <- CARD_deconvolution(
#'     sc_count = sc_count,
#'     sc_meta = sc_meta,
#'     spatial_count = spatial_count,
#'     spatial_location = spatial_location,
#'     ct_varname = "cellType",
#'     ct_select = unique(sc_meta$cellType),
#'     sample_varname = "sampleInfo",
#'     mincountgene = 100,
#'     mincountspot = 5
#' )
#' CARD_visualize_gene(
#'     spatial_expression = assays(CARD_obj)$spatial_countMat,
#'     spatial_location = spatialCoords(CARD_obj),
#'     gene_visualize = c("A4GNT", "AAMDC", "CD248"),
#'     colors = NULL,
#'     NumCols = 3
#' )
#'
CARD_visualize_gene <- function(
        spatial_expression, 
        spatial_location,
        gene_visualize, 
        colors = colors, 
        NumCols) {
    #####Preapare dataframe for plotting
    expression <- as.data.frame(as.matrix(spatial_expression))
    expression <- sweep(expression, 2, colSums(expression), "/")
    location <- as.data.frame(spatial_location)
    if (sum(colnames(expression) == rownames(location)) != nrow(location)) {
        stop("The colnames of expression data does not match with the rownames
        of spatial location data")
    }
    gene_select <- gene_visualize
    if (sum(toupper(gene_select) %in% toupper(rownames(spatial_expression))) !=
        length(gene_select)) {
        stop("There existing selected genes that are not in the 
            expression data!")
    }
    plotdata <- NULL
    for (i in seq_len(length(gene_select))) {
        #### load spatial dataset
        gene <- gene_select[i]
        ind <- which(toupper(rownames(expression)) == toupper(gene))
        df <- as.numeric(expression[ind, ])
        names(df) <- colnames(expression)
        df <- (df - min(df)) / (max(df) - min(df))
        d <- data.frame(
            value = df,
            x = as.numeric(location$x),
            y = as.numeric(location$y)
        )
        d$gene <- gene
        plotdata <- rbind(plotdata, d)
    }
    plotdata$gene <- factor(plotdata$gene, levels = gene_select)
    
    ##### Plot
    p <- ggplot(plotdata, aes(x, y)) +
        geom_point(
            aes(color = value),
            size = 1.5,
            shape = 15,
            position = position_dodge2(width = 0.05)
        ) +
        scale_x_discrete(expand = c(0, 1)) +
        scale_y_discrete(expand = c(0, 1)) +
        coord_equal() +
        facet_wrap(~gene, ncol = NumCols) +
        theme(
            plot.margin = margin(0.1, 0.1, 0.1, 0.1, "cm"),
            legend.position = "bottom",
            panel.background = element_blank(),
            plot.background = element_blank(),
            panel.border = element_rect(
                colour = "grey89", 
                fill = NA, 
                linewidth = 0.5),
            axis.text = element_blank(),
            axis.ticks = element_blank(),
            axis.title = element_blank(),
            legend.title = element_text(size = 15, face = "bold"),
            legend.text = element_text(size = 14),
            strip.text = element_text(size = 18, face = "bold"),
            legend.key = element_rect(colour = "transparent", fill = "white"),
            legend.key.size = unit(1.0, "cm")
        ) +
        guides(color = guide_colourbar(title = "Expression"))
    if (is.null(colors)) {
        p <- p + scale_color_viridis_c(
            labels = c("0", "0.25", "0.5", "0.75", "1.0"))
    } else {
        p <- p + scale_color_gradientn(colours = colors)
    }
    return(p)
}

test_that("Visulization for each cell type works well", {
    ## use global reference-based deconvolution result
    expect_no_error({
        p1 <- CARD_visualize_pie(
            proportion = CARD_obj$Proportion_CARD,
            spatial_location = spatialCoords(CARD_obj), 
            colors = NULL, 
            radius = NULL)
    })
})


test_that("Visulization foe selected cell types works well", {
    ## use global reference-based deconvolution result
    ct.visualize = c("Acinar_cells","Cancer_clone_B",
                     "Ductal_terminal_ductal_like",
                     "Ductal_CRISP3_high-centroacinar_like",
                     "Ductal_MHC_Class_II","Ductal_APOL1_high-hypoxic")
    ## visualize the spatial distribution of the cell type proportion
    expect_no_error({
        p2 <- CARD_visualize_prop(
            proportion = CARD_obj$Proportion_CARD,        
            spatial_location = spatialCoords(CARD_obj), 
            ct_visualize = ct.visualize,                 
            colors = c("lightblue","lightyellow","red"), 
            NumCols = 4,                                 
            pointSize = 3.0) 
    })
})

test_that("Visualization for two cell types works well", {
    ## use global reference-based deconvolution result
    ## visualize the spatial distribution of the cell type proportion
    expect_no_error({
        p3 <- CARD_visualize_prop_2CT(
            proportion = CARD_obj$Proportion_CARD,                          
            spatial_location = spatialCoords(CARD_obj),                     
            ct2_visualize = c("Acinar_cells","Cancer_clone_B"),              
            colors = list(c("lightblue","lightyellow","red"),
                          c("lightblue","lightyellow","black")))
    })
})

test_that("Visualization for correlation works well", {
    ## use global reference-based deconvolution result
    expect_no_error({
        p4 <- CARD_visualize_Cor(CARD_obj$Proportion_CARD,colors = NULL)
    })
})


test_that("Visualization for genes works well", {
    ## use global reference-based deconvolution result
    expect_no_error({
        p7 <- CARD_visualize_gene(
            spatial_expression = assays(CARD_obj)$spatial_countMat,
            spatial_location = spatialCoords(CARD_obj),
            gene_visualize = c("A4GNT", "AAMDC", "CD248"),
            colors = NULL,
            NumCols = 3)
    })
})

#' The function to estimate the cell type composition signature for each
#' single cell in the scRNaseq reference data
#'
#' @param sc_eset the sc_eset stored in the CARD object
#' @param ct_varname character, the name of the column in metaData that
#' specifies the cell type annotation information, stored in the CARD object
#' @param ct_select vector of cell type names that you are interested in to
#' deconvolute, default as NULL. stored in the CARD object
#' @param sample_varname character,the name of the column in metaData that
#' specifies the sample information. stored in the CARD object
#' @param B reference basis matrix stored in the CARD object.

#' @importFrom SummarizedExperiment assays
#' @importFrom nnls nnls

#' @return Returns a matrix of the cell type composition signature for each
#' single cell in the scRNaseq reference
#'
get_weight_for_cell <- function(
        sc_eset, 
        ct_varname, 
        ct_select, 
        sample_varname, 
        B){
    ##### Calculate cell type composition signature matrix
    count <- assays(sc_eset)$counts
    count <- count[, colSums(count) > 0]
    count <- sweep(count, 2, colSums(count), "/")
    count <- count[rownames(count) %in% rownames(B), ]
    count <- count[match(rownames(B), rownames(count)), ]
    mean_cell <- vapply(seq_len(ncol(count)), function(icell) {
        mod1 <- nnls(as.matrix(B), as.matrix(count[, icell]))
        mod1$x
    }, FUN.VALUE = numeric(ncol(B)))
    mean_cell <- t(mean_cell)
    rownames(mean_cell) <- colnames(count)
    colnames(mean_cell) <- colnames(B)
    return(mean_cell)
    }
#' The function to sample the spatial location information for each single
#' cell
#'
#' @param cords The spatial location information in the measure spatial
#' locations, with the first and second columns represent the 2-D 
#' x-y coordinate system
#' @param numcell a numeric value indicating the number of single cells in each
#' measured location, we suggest 20 for ST technology, 7 for 10x Viisum and 2
#' for Slide-seq
#' @param shape a character indicating whether the sampled spatial coordinates
#' for single cells locating in a Square-like region or a Circle-like region.
#' The center of this region is the measured spatial location in the non-single
#' cell resolution spatial transcriptomics data. The default is 'Square', the
#' other shape is 'Circle'
#' @importFrom spatstat.random runifdisc
#' @importFrom fields rdist
#' @importFrom stats runif median
#' @return Returns a dataframe with the sampled spatial location information
#' for each single cell
#'
get_high_res_cords <- function(cords, numcell, shape = "Square") {
    ##### Return sampled spatial location information
    ed <- rdist(as.matrix(cords))
    n <- dim(cords)[1]
    dis <- c()
    for (i in seq_len(dim(ed)[1])) {
        dis[i] <- min(ed[i, -i])
    }
    min_distance <- median(dis) / 2
    cords_new <- NULL
    for (i in seq_len(nrow(cords))) {
        get_points_within_circle <- function(ed, i, numcell, min_distance) {
            circle <- runifdisc(
                numcell,
                radius = min_distance,
                centre = c(cords[i, 1], cords[i, 2]),
                nsim = 1,
                drop = TRUE
            )
            df <- data.frame(x = circle$x, y = circle$y)
            return(df)
        }
        get_points_within_square <- function(cords, i, numcell, min_distance) {
            minX <- min_distance
            minY <- min_distance
            rectangular_x <- runif(
                numcell,
                min = cords[i, 1] - minX,
                max = cords[i, 1] + minX
            )
            rectangular_y <- runif(
                numcell,
                min = cords[i, 2] - minY,
                max = cords[i, 2] + minY
            )
            df <- data.frame(x = rectangular_x, y = rectangular_y)
            return(df)
        }
        if (shape == "Square") {
            df <- get_points_within_square(cords, i, numcell, min_distance)
        } else if (shape == "Circle") {
            df <- get_points_within_circle(ed, i, numcell, min_distance)
        }
        colnames(df) <- c("x", "y")
        df$centerSPOT <- paste0(cords[i, 1], "x", cords[i, 2])
        df$centerx <- cords[i, 1]
        df$centery <- cords[i, 2]
        cords_new <- rbind(cords_new, df)
    }
    cords_new <- cords_new[!duplicated(paste0(cords_new$x, "x", cords_new$y)), ]
    rownames(cords_new) <- paste0(cords_new$x, "x", cords_new$y)
    return(cords_new)
}

#' The function to assign the spatial location information for
#' each single cell
#'
#' @param mappint_spot_cell_cor a mapped correlation matrix indicating the
#' relashionship between each measured spatial location and the single cell in
#' the scRNAseq reference
#' @param cords_new output from the function get_high_res_cords
#' @param numcell a numeric value indicating the number of single cells in each
#' measured location, we suggest 20 for ST technology, 7 for 10x Viisum and 2
#' for Slide-seq
#' @param sc_eset a single cell experiment object stored in CARD object
#' @param ct_varname character, the name of the column in metaData that
#' specifies the cell type annotation information, stroed in CARD object

#' @importFrom SingleCellExperiment colData
#' @return Return the assigned spatial location information for the mapped
#' single cell
#'
assign_sc_cords <- function(
        mappint_spot_cell_cor, 
        cords_new, 
        numcell,
        sc_eset, 
        ct_varname) {
    ##### Assign spatial information for mapping single cell
    mapcellcords <- NULL
    for (ispot in seq_len(nrow(mappint_spot_cell_cor))) {
        mapcell <- mappint_spot_cell_cor[ispot, ]
        mapcell <- mapcell[order(mapcell, decreasing = TRUE)]
        centerspot <- rownames(mappint_spot_cell_cor)[ispot]
        ispot_cords <- data.frame(
            x = as.numeric(
                vapply(
                    strsplit(centerspot, split = "x"), 
                    `[`, 
                    character(1),
                    1)
                ),
            y = as.numeric(
                vapply(
                    strsplit(centerspot, split = "x"), 
                    `[`, 
                    character(1), 
                    2)
                )
        )
        subcordscell <- cords_new[cords_new$centerSPOT == centerspot, ]
        
        ##### calculate Euclidiean distance with this
        ed_with_center <- rdist(
            as.matrix(subcordscell[, seq_len(2)]),
            as.matrix(ispot_cords)
        )
        subcordscell$ed_with_center <- ed_with_center
        subcordscell <- subcordscell[
            order(subcordscell$ed_with_center,decreasing = FALSE), ]
        map_cell_cords_temp <- data.frame(
            CorwithSpot = mapcell[seq_len(numcell)])
        map_cell_cords_temp <- cbind(map_cell_cords_temp, subcordscell)
        map_cell_cords_temp$CT <- 
            colData(sc_eset)[rownames(map_cell_cords_temp),ct_varname]
        map_cell_cords_temp$Cell <- rownames(map_cell_cords_temp)
        mapcellcords <- rbind(mapcellcords, map_cell_cords_temp)
    }
    return(mapcellcords)
}

#' Extension of CARD into performing single cell Mapping from non-single
#' cell spatial transcriptomics dataset.
#'
#' @param CARD_object CARD object create by the CARD_deconvolution function.
#' @param shapeSpot a character indicating whether the sampled spatial
#' coordinates for single cells locating in a Square-like region or a
#' Circle-like region. The center of this region is the measured spatial
#' location in the non-single cell resolution spatial transcriptomics data.
#' The default is 'Square', the other shape is 'Circle'
#' @param numcell a numeric value indicating the number of single cells in
#' each measured location, we suggest 20 for ST technology, 7 for 10x Viisum
#' and 2 for Slide-seq
#' @param ncore a numeric value indicating the number of cores used to
#' accelerating the procedure

#' @importFrom SummarizedExperiment assays colData
#' @importFrom BiocParallel bplapply MulticoreParam SnowParam
#' @return Returns a SingleCellExperiment SCE object with the mapped expression
#' at single cell resolution and the spatial location information of each
#' single cell
#'
#' @export
#' @examples
#' library(SingleCellExperiment)
#' data(spatial_count)
#' data(spatial_location)
#' data(sc_count)
#' data(sc_meta)
#' CARD_obj <- CARD_deconvolution(
#'     sc_count = sc_count,
#'     sc_meta = sc_meta,
#'     spatial_count = spatial_count,
#'     spatial_location = spatial_location,
#'     ct_varname = "cellType",
#'     ct_select = unique(sc_meta$cellType),
#'     sample_varname = "sampleInfo",
#'     mincountgene = 100,
#'     mincountspot = 5
#' )
#' scMapping <- CARD_scmapping(
#' CARD_obj, 
#' shapeSpot = "Square", 
#' numcell = 20, 
#' ncore = 2)
#' print(scMapping)
#'
CARD_scmapping <- function(
        CARD_object, 
        shapeSpot = "Square",
        numcell, 
        ncore = 10) {
    #####Check
    if(shapeSpot != 'Square' && shapeSpot != 'Circle'){
        stop("Please use 'Square' or 'Circle'")
    }
    if (!inherits(CARD_object, "SpatialExperiment")) {
        stop("CARD_object must be a SpatialExperiment object is created by
        CARD functions")
    }
    if (!is.numeric(ncore) || 
        length(ncore) != 1 || 
        ncore <= 0 || 
        ncore != as.integer(ncore)) {
        stop("ncore must be a single positive integer")
    }
    if (!is.numeric(numcell) || 
        length(numcell) != 1 || 
        numcell <= 0 || 
        numcell != as.integer(numcell)) {
        stop("numcell must be a single positive integer")
    }
    
    ##### load in spatial transcriptomics data stored in CARD_object
    sc_eset <- CARD_object@metadata$sc_eset
    B <- CARD_object@metadata$algorithm_matrix$B
    ct_select <- CARD_object@metadata$info_parameters$ct.select
    ct_varname <- CARD_object@metadata$info_parameters$ct.varname
    sample_varname <- CARD_object@metadata$info_parameters$sample.varname
    res_CARD <- if (!is.null(CARD_object$Proportion_CARD)) {
        CARD_object$Proportion_CARD
    } else {
        CARD_object@metadata$Proportion_CARD
    }
    res_CARD <- res_CARD[, order(colnames(res_CARD))]
    mean_cell <- get_weight_for_cell(
        sc_eset, 
        ct_varname, 
        ct_select,
        sample_varname, 
        B
    )
    mean_cell <- mean_cell[, order(colnames(mean_cell))]
    cords <- if (dim(spatialCoords(CARD_object))[2] > 0) {
        as.data.frame(spatialCoords(CARD_object)[rownames(res_CARD),])
    } else {
        as.data.frame(
            CARD_object@metadata$spatial_location[rownames(res_CARD), ])
    }
    rownames(res_CARD) <- paste0(cords$x, "x", cords$y)
    cords_new <- get_high_res_cords(
        cords,
        numcell = numcell,
        shape = shapeSpot
    )
    mappint_spot_cell_cor <- cor(t(res_CARD), t(mean_cell))
    rownames(mappint_spot_cell_cor) <- 
        paste0(
            cords$x, 
            "x",
            cords$y
            )
    
    ##### high resolution coordinates
    mapcellcords <- assign_sc_cords(
        mappint_spot_cell_cor, 
        cords_new,
        numcell, 
        sc_eset, 
        ct_varname
    )
    
    ##### map scRNAseq count data
    count_sc <- assays(sc_eset)$counts
    count_sc <- count_sc[, colSums(count_sc) > 0]
    count_ct <- NULL
    
    ##### parallelization parameter
    param <- if (.Platform$OS.type == "windows") {
        SnowParam(workers = ncore)
    } else {
        MulticoreParam(workers = ncore)
    }
    count_ct <- bplapply(seq_len(nrow(res_CARD)), function(ispot) {
        spot <- rownames(res_CARD)[ispot]
        map_cell_cords_spot <- mapcellcords[mapcellcords$centerSPOT == spot, ]
        df <- as(count_sc[, map_cell_cords_spot$Cell], "sparseMatrix")
        colnames(df) <- paste0(map_cell_cords_spot$Cell, ":", spot)
        colnames(df) <- paste0(
            colnames(df), 
            ":", 
            map_cell_cords_spot$x,
            "x", 
            map_cell_cords_spot$y
        )
        df
        }, BPPARAM = param)
    count_ct <- do.call("cbind", count_ct)
    rownames(mapcellcords) <- colnames(count_ct)
    sce <- SingleCellExperiment(
        list(counts = count_ct),
        colData = as.data.frame(mapcellcords[, c(
            "x", 
            "y", 
            "centerSPOT",
            "centerx", 
            "centery",
            "CT", 
            "Cell")]),
        rowData = as.data.frame(rownames(count_ct))
    )
    return(sce)
}

#' Each CARD object has a number of slots which store information. Key
#' slots to access are listed below.
#'
#' @slot sc_eset The filtered scRNA-seq data along with meta data stored
#' in the format of SingleCellExperiment.
#' @slot spatial_countMat The filtered spatial count data.
#' @slot spatial_location The weights for combining p-values from multiple
#' kernels.
#' @slot Proportion_CARD The estimated cell type proportion by
#' CARD with each row is a spatial location and each column is a
#' cell type.
#' @slot project The name of the project, default is deconvolution.
#' @slot info_parameters The paramters that are used in model fitting.
#' @slot algorithm_matrix The intermediate matrices that are used in
#' the model fitting step.
#' @slot refined_prop The refined cell type proportion matrix estimated
#' by CARD for the newly grided spatial locations. The number of
#' initial grids are defined by the user.
#' @slot refined_expression The refined predicted expression matrix
#' (normalized) estimated by CARD for the newly grided spatial locations.
#' The number of initial grids are defined by the user.
#'
#' @return Return an object of CARD class
setClass("CARD",
        slots = list(
            sc_eset = "ANY",
            spatial_countMat = "ANY",
            spatial_location = "data.frame",
            Proportion_CARD = "matrix",
            project = "character",
            info_parameters = "list",
            algorithm_matrix = "list",
            refined_prop = "matrix",
            refined_expression = "matrix"
        ))


#' Each CARDfree object has a number of slots which store information.
#' Key slots to access are listed below.
#'
#' @slot spatial_countMat The filtered spatial count data.
#' @slot spatial_location The weights for combining p-values from multiple
#' kernels.
#' @slot Proportion_CARD The estimated cell type proportion by CARD
#' with each row is a spatial location and each column is a cell type.
#' @slot estimated_refMatrix The estimated reference matrix by CARDfree
#' with each row represents a gene and each column represents a cell type
#' cluster.
#' @slot project The name of the project, default is deconvolution.
#' @slot markerList The nlist of cell type specific markers, with
#' each element represents the vector of cell type specific markers
#' @slot info_parameters The paramters that are used in model fitting.
#' @slot algorithm_matrix The intermediate matrices that are used in
#' the model fitting step.
#' @slot refined_prop The refined cell type proportion matrix estimated
#' by CARD for the newly grided spatial locations. The number of initial
#' grids are defined by the user.
#' @slot refined_expression The refined predicted expression matrix
#' (normalized) estimated by CARD for the newly grided spatial locations.
#' The number of initial grids are defined by the user.
#'
#' @return Return an object of CARDfree class
setClass("CARDfree",
        slots = list(
            spatial_countMat = "ANY",
            spatial_location = "data.frame",
            Proportion_CARD = "matrix",
            estimated_refMatrix = "matrix",
            project = "character",
            markerList = "list",
            info_parameters = "list",
            algorithm_matrix = "list",
            refined_prop = "matrix",
            refined_expression = "matrix"
        ))


#' Show method for the CARD class
#'
#' This method provides a concise summary of an object of class \code{CARD}, 
#' displaying key information including the project name, the number of spots,
#' the number of cell types, and a sample of the 
#' \code{Proportion_CARD} matrix.
#'
#' @param object An object of class \code{CARD}.
#' 
#' @return A concise summary of the \code{CARD} object is printed to the 
#' console.
#' 


setMethod("show", "CARD", function(object) {
    cat("An object of class 'CARD'\n")
    cat("\nProject: ", object@project, "\n")
    cat("Number of spots: ", dim(object@spatial_countMat)[2], "\n")
    cat("Number of cell types: ",length(unique(object@sc_eset$cellType)), "\n")
})



#' Show method for the CARDfree class
#'
#' This method provides a concise summary of an object of 
#' class \code{CARDfree}, displaying key information including the project 
#' name, the number of spots, the number of cell types, and a sample of the 
#' \code{Proportion_CARD} matrix.
#'
#' @param object An object of class \code{CARDfree}.
#' 
#' @return A concise summary of the \code{CARDfree} object is 
#' printed to the console.
#' 

setMethod("show", "CARDfree", function(object) {
    cat("An object of class 'CARDfree'\n")
    cat("\nProject: ", object@project, "\n")
    cat("Number of spots: ", dim(object@spatial_countMat)[2], "\n")
})

#' Quality control of scRNA-seq count data
#'
#' @param counts_in Raw scRNAseq count data, each column is a cell and each
#' row is a gene.
#' @param metadata data frame, metadata with "ct_varname" specify the cell type
#' annotation information and "sample_varname" specify the sample information
#' @param ct_varname character, the name of the column in metadata that
#' specifies the cell type annotation information
#' @param ct_select vector of cell type names that you are interested in to
#' deconvolute, default as NULL. If NULL, then use all cell types provided by
#' single cell dataset;
#' @param sample_varname character,the name of the column in metadata that
#' specifies the sample information. If NULL, we just use the whole as one
#' sample.
#' @param min_cells numeric, we filtered out the non-expressed cells.
#' @param min_genes numeric we filtered out the non-expressed genes
#'
#' @importFrom SingleCellExperiment SingleCellExperiment
#' @return Return the filtered scRNA-seq data and meta data stored in a S4 class
#' (SingleCellExperiment)
#'

sc_QC <- function(
        counts_in, 
        metadata, 
        ct_varname, 
        ct_select,
        sample_varname = NULL, 
        min_cells = 0, 
        min_genes = 0) {
    ##### Filter based on min.features
    coldf <- metadata
    counts <- counts_in
    if (min_genes >= 0) {
        nfeatures <- colSums(x = counts)
        counts <- counts[, which(x = nfeatures > min_genes)]
        coldf <- coldf[which(x = nfeatures > min_genes), ]
    }
    
    ##### filter genes on the number of cells expressing
    if (min_cells >= 0) {
        num.cells <- rowSums(x = counts > 0)
        counts <- counts[which(x = num.cells > min_cells), ]
    }
    fdata <- as.data.frame(rownames(counts))
    rownames(fdata) <- rownames(counts)
    keep_cell <- as.character(coldf[, ct_varname]) %in% ct_select
    counts <- counts[, keep_cell]
    coldf <- coldf[keep_cell, ]
    keep_gene <- rowSums(counts) > 0
    fdata <- as.data.frame(fdata[keep_gene, ])
    counts <- counts[keep_gene, ]
    sce <- SingleCellExperiment(
        list(counts = counts),
        colData = as.data.frame(coldf),
        rowData = as.data.frame(fdata)
    )
    return(sce)
}

#' Create the CARD object
#'
#' @param sc_count Raw scRNA-seq count data, each column is a cell and each row
#' is a gene.
#' @param sc_meta data frame, with each row representing the cell type and/or
#' sample information of a specific cell. The row names of this data frame
#' should match exactly with the column names of the sc_count data
#' @param spatial_count Raw spatial resolved transcriptomics data, each column
#' is a spatial location, and each row is a gene.
#' @param spatial_location data frame, with two columns representing the x and
#' y coordinates of the spatial location. The rownames of this data frame
#' should match eaxctly with the columns of the spatial_count.
#' @param sce a \code{SingleCellExperiment} object containing scRNA-seq count 
#' data in the \code{counts} assay, and cell types and sample information in 
#' the colData.
#' @param spe a \code{SpatialExperiment} object containing spatial data in 
#' the \code{counts} assay, and spatial coordinates in the spatialCoords.
#' @param ct_varname character, the name of the column in metadata that
#' specifies the cell type annotation information
#' @param ct_select vector of cell type names that you are interested in to
#' deconvolute, default as NULL. If NULL, then use all cell types provided by
#' single cell dataset;
#' @param sample_varname character,the name of the column in metadata that
#' specifies the sample information. If NULL, we just use the whole as one
#' sample.
#' @param mincountgene Minimum counts for each gene
#' @param mincountspot Minimum counts for each spatial location
#'
#' @importFrom SummarizedExperiment assays
#' @import methods
#' @import SingleCellExperiment
#' @import SpatialExperiment
#' @return Returns CARD object with filtered spatial count and single cell
#' RNA-seq dataset.
#'
createCARDObject <- function(
        sc_count, 
        sc_meta, 
        spatial_count, 
        spatial_location,
        ct_varname, 
        ct_select, 
        sample_varname,
        mincountgene = 100, 
        mincountspot = 5,
        sce = NULL, 
        spe = NULL) {
    ##### Extract count and spatial information
    if(is.null(sc_count) || 
        is.null(sc_meta) || 
        is.null(spatial_count) || 
        is.null(spatial_location)){
        if(!is.null(sce) && !is.null(spe)){
            sc_count <- assays(sce)[['counts']]
            sc_meta <- as.data.frame(sce@colData)
            spatial_count <- assays(spe)[['counts']]
            spatial_location <- as.data.frame(spatialCoords(spe))
        } else{
            stop("Please provide SingleCellExperiment object and 
            SpatialExperiment object")
        }
    }
    
    ##### QC on scRNASeq dataset
    message("## QC on scRNASeq dataset! ...\n")
    if (is(sc_count, "matrix")) {
        sc_countmat <- as(as.matrix(sc_count), "sparseMatrix")
    } else if (is(sc_count, "vector")) {
        sc_countmat <- as(t(as.matrix(sc_count)), "sparseMatrix")
    } else if (is(sc_count, "sparseMatrix")) {
        sc_countmat <- sc_count
    } else {
        stop("scRNASeq counts has to be of following forms: vector,
        matrix or sparseMatrix")
    }
    if (missing(x = sc_countmat)) {
        stop("Please provide scRNASeq count data")
    } else if (is.null(sample_varname) || missing(sample_varname)) {
        sample_varname <- "sampleID"
        sc_meta <- as.data.frame(sc_meta)
        sc_meta$sampleID <- "Sample"
    } else if (any(rownames(x = sc_countmat) == "")) {
        stop("Feature names of sc_count matrix cannot be empty",
            call. = FALSE
        )
    } else if (sum(rownames(sc_meta) == colnames(sc_countmat)) !=
            ncol(sc_countmat)) {
        stop("Cell name in scRNAseq count data does not match with
        the rownames of metadata")
    } else if (ncol(sc_countmat) != nrow(sc_meta)) {
        stop("The number of cells in scRNA-seq counts and sc_meta
            should be consistent!
            (sc_count -- p x c; sc_meta -- c x 2)")
    }
    if (is.null(ct_varname)) {
        stop("Please provide the column name indicating the cell type 
        information in the meta data of scRNA-seq")
    } else if (is.null(ct_select)) {
        message("No cell types selected, we will use all the cell types in the
                scRNA-seq data\n")
        ct_select <- unique(sc_meta[, ct_varname])
    }
    ct_select <- as.character(ct_select[!is.na(ct_select)])
    sc_eset <- sc_QC(
        sc_countmat, 
        sc_meta, 
        ct_varname, 
        ct_select, 
        sample_varname)
    
    ##### Check the spatial count dataset
    #### QC on spatial dataset
    message("## QC on spatially-resolved dataset! ...\n")
    if (is(spatial_count, "matrix")) {
        spatial_countmat <- as(as.matrix(spatial_count), "sparseMatrix")
    } else if (is(spatial_count, "vector")) {
        spatial_countmat <- as(t(as.matrix(spatial_count)), "sparseMatrix")
    } else if (is(spatial_count, "sparseMatrix")) {
        spatial_countmat <- spatial_count
    } else {
        stop("spatial resolved transcriptomic counts has to be of
                    following forms: vector,matrix or sparseMatrix")
    }
    if (any(rownames(x = spatial_countmat) == "")) {
        stop("Gene names of spatial count matrix cannot be empty",
            call. = FALSE
        )
    }
    commongene <- intersect(
        rownames(spatial_countmat),
        rownames(assays(sc_eset)$counts)
    )
    if (length(commongene) == 0) {
        stop("There are no common gene names in spatial count data and
                single cell RNAseq count data", call. = FALSE)
    }
    if (is.null(spatial_location)) {
        stop("Please provide the matched spatial location data frame")
    }
    if (ncol(spatial_countmat) != nrow(spatial_location)) {
        stop("The number of spatial locations in spatial_count and 
        spatial_location should be consistent!
        (spatial_count -- p x n; spatial_location -- n x 2)")
    } # end fi
    ## check data order should consistent
    if (!identical(colnames(spatial_countmat), rownames(spatial_location))) {
        stop("The column names of spatial_count and row names of 
        spatial_location should be should be matched each other!
        (spatial_count -- p x n; spatial_location -- n x 2)")
    } # end fi
    
    ##### QC on spatial dataset
    spatial_countmat <- spatial_countmat[rowSums(spatial_countmat > 0) >
        mincountspot, ]
    spatial_countmat <- 
        spatial_countmat[, (colSums(spatial_countmat) >= mincountgene &
                                colSums(spatial_countmat) <= 1e6)]
    spatial_location <- spatial_location[rownames(spatial_location) %in%
        colnames(spatial_countmat), ]
    spatial_location <- spatial_location[
        match(colnames(spatial_countmat),rownames(spatial_location)), ]
    object <- new(
        Class = "CARD",
        sc_eset = sc_eset,
        spatial_countMat = spatial_countmat,
        spatial_location = spatial_location,
        project = "Deconvolution",
        info_parameters = list(
            ct.varname = ct_varname, 
            ct.select = ct_select,
            sample.varname = sample_varname
        )
    )
    return(object)
}

#' Create the CARD object
#'
#' @param markerlist a list of marker genes, with each element of the list
#' being the vector of cell type specific marker genes
#' @param spatial_count Raw spatial resolved transcriptomics data, each column
#' is a spatial location, and each row is a gene.
#' @param spatial_location data frame, with two columns representing the x and
#' y coordinates of the spatial location. The rownames of this data frame
#' should match eaxctly with the columns of the spatial_count.
#' @param spe a \code{SpatialExperiment} object containing spatial data in 
#' the \code{counts} assay, and spatial coordinates in the spatialCoords.
#' @param mincountgene Minimum counts for each gene
#' @param mincountspot Minimum counts for each spatial location
#'
#' @import methods
#' @import SpatialExperiment
#' @return Returns CARDfree object with filtered spatial count and marker gene
#' list.
#'

createCARDfreeObject <- function(
        markerlist, 
        spatial_count, 
        spatial_location,
        mincountgene = 100, 
        mincountspot = 5,
        spe = NULL) {
    ##### Extract count and spatial information
    if(is.null(spatial_count) || is.null(spatial_location)){
        if(!is.null(spe)){
            spatial_count <- assays(spe)[['counts']]
            spatial_location <- as.data.frame(spatialCoords(spe))
        }else{
            stop("Please provide SpatialExperiment object")
        }
    }
    if (is(spatial_count, "matrix")) {
        spatial_countmat <- as(as.matrix(spatial_count), "sparseMatrix")
    } else if (is(spatial_count, "vector")) {
        spatial_countmat <- as(t(as.matrix(spatial_count)), "sparseMatrix")
    } else if (is(spatial_count, "sparseMatrix")) {
        spatial_countmat <- spatial_count
    } else {
        stop("spatial resolved transcriptomic counts has to be of
        following forms: vector,matrix or sparseMatrix")
    } # end fi
    if (any(rownames(x = spatial_countmat) == "")) {
        stop("Gene names of spatial count matrix cannot be empty",
            call. = FALSE
        )
    } # end fi
    if (is.null(spatial_location)) {
        stop("Please provide the matched spatial location data frame")
    } # end fi
    if (ncol(spatial_countmat) != nrow(spatial_location)) {
        stop("The number of spatial locations in spatial_count and 
        spatial_location should be consistent!
        (spatial_count -- p x n; spatial_location -- n x 2)")
    } # end fi
    ## check data order should consistent
    if (!identical(colnames(spatial_countmat), rownames(spatial_location))) {
        stop("The column names of spatial_count and row names of 
        spatial_location should be should be matched each other!
        (spatial_count -- p x n; spatial_location -- n x 2)")
    } # end fi
    
    ##### QC on spatial dataset
    spatial_countmat <- spatial_countmat[rowSums(spatial_countmat > 0) >
        mincountspot, ]
    spatial_countmat <- spatial_countmat[, (colSums(spatial_countmat) >= 
                                                mincountgene &
        colSums(spatial_countmat) <= 1e6)]
    spatial_location <- spatial_location[rownames(spatial_location) %in%
        colnames(spatial_countmat), ]
    spatial_location <- spatial_location[match(
        colnames(spatial_countmat),
        rownames(spatial_location)
    ), ]
    #### check marker gene list
    if (missing(x = markerlist)) {
        stop("Please provide the marker list for CARDfree")
    }
    object <- new(
        Class = "CARDfree",
        spatial_countMat = spatial_countmat,
        spatial_location = spatial_location,
        project = "Deconvolution (reference-free)",
        markerList = markerlist,
        info_parameters = list()
    )
    return(object)
}

#' scRNAseq meta data
#'
#' The scRNAseq meta data must be in the format of data frame while each row
#' represents a cell. The rownames of the scRNAseq meta data should match
#' exactly with the column names of the scRNAseq count data. The sc_meta data
#' must contain the column indicating the cell type assignment for each cell
#' (e.g., “cellType” column in the example sc_meta data). Sample/subject
#' information should be provided, if there is only one sample, we can add a
#' column by sc_meta$sampleInfo = "sample1".
#'
#'
#' @docType data
#' @keywords datasets
#' @usage data(sc_meta)
#' @name sc_meta
"sc_meta"

#' scRNA-seq count data
#'
#' The scRNA-seq count data must be in the format of matrix or sparseMatrix,
#' while each row represents a gene and each column represents a cell.
#'
#'
#' @docType data
#' @keywords datasets
#' @usage data(sc_count)
#' @name sc_count
"sc_count"

#' Spatial transcriptomics count data
#'
#' The spatial transcriptomics count data must be in the format of matrix or
#' sparseMatrix, while each row represents a gene and each column represents a
#' spatial location. The column names of the spatial data can be in the
#' “XcoordxYcoord” (i.e., 10x10) format, but you can also maintain your original
#' spot names, for example, barcode names.
#'
#'
#' @docType data
#' @keywords datasets
#' @usage data(spatial_count)
#' @name spatial_count
"spatial_count"

#' Make new spatial locations on unmeasured tissue through grids.
#'
#' @param location Data frame, spatial location data frame of the original
#' spatial resolved transcriptomics dataset, stored in the
#' spatialCoords(CARD_object)
#' @param num_sample Numeric, approximate number of cells in grid within the
#' shape of the spatial location data frame
#' @param concavity Numeric, a relative measure of concavity. The default is
#' 2.0, which can prodecure detailed enough shapes. Infinity results in a
#' convex hull while 1 results in a more detailed shape.
#'
#' @import concaveman
#' @import sp
#' @importFrom dplyr '%>%'
#' @importFrom sf st_as_sf
#' @return Return a list of data frame with newly grided points
#'
#'

sample_grid_within <- function(location, num_sample, concavity = 2) {
    #### recognise the outline of the shape and extract the coordinates of
    ##### the polygons
    df <- location[, seq_len(2)]
    pnts <- df %>%
        st_as_sf(coords = c("x", "y"))
    polygon <- concaveman(pnts, concavity) #### tune the number, default is 2.0
    poly_coords <- as.data.frame(as.matrix(polygon$polygons[[1]]))
    colnames(poly_coords) <- c("x", "y")
    coordinates(poly_coords) <- ~ x + y
    
    ##### create the spatial polygon data frame
    crds <- poly_coords
    pl <- Polygon(crds)
    id <- "21*15"
    pls <- Polygons(list(pl), ID = id)
    spls <- SpatialPolygons(list(pls))
    df <- data.frame(value = 1, row.names = id)
    spdf <- SpatialPolygonsDataFrame(spls, df)
    
    ##### random make the grids within the polygon
    pts <- makegrid(spdf, num_sample)
    pts1 <- SpatialPoints(pts)
    
    ##### extract the points within the locations
    spgrd_within <- SpatialPixels(pts1[spdf, ])
    data <- as.data.frame(spgrd_within@coords)
    data$id <- seq_len(nrow(data))
    colnames(data) <- c("x", "y")
    return(data)
}

#' Normalize the new spatial locations without changing the shape and
#' relative positions
#'
#' @param location_orig Data frame, spatial location data frame of the original
#' spatial resolved transcriptomics dataset, stored in the
#' spatialCoords(CARD_object)
#' @param train_ind Vector, Index of the original spatial locations
#' @param test_ind Vector, Index of the newly grided spatial locations
#'
#' @return Return the normalized spatial location data frame
#'

norm_coords_train_test <- function(location_orig, train_ind, test_ind) {
    ##### normalize to 0-1 scale
    norm_coords_train <- as.data.frame(location_orig[train_ind, seq_len(2)])
    location_factor_x <- min(norm_coords_train$x)
    location_factor_y <- min(norm_coords_train$y)
    norm_coords_train$x <- norm_coords_train$x - location_factor_x
    norm_coords_train$y <- norm_coords_train$y - location_factor_y
    scale_factor <- max(norm_coords_train$x, norm_coords_train$y)
    norm_coords_train$x <- norm_coords_train$x / scale_factor
    norm_coords_train$y <- norm_coords_train$y / scale_factor
    norm_coords_test <- as.data.frame(location_orig[test_ind, seq_len(2)])
    norm_coords_test[, 1] <- (norm_coords_test[, 1] -
        location_factor_x) / scale_factor
    norm_coords_test[, 2] <- (norm_coords_test[, 2] -
        location_factor_y) / scale_factor
    locationTemp <- rbind(norm_coords_test, norm_coords_train)
    return(locationTemp)
}

#' Calculate the variance covariance matrix used in the imputation of the
#' new grided locations
#'
#' @param location_orig Data frame, spatial location data frame of the original
#' spatial resolved transcriptomics dataset, stored in the
#' spatialCoords(CARD_object)
#' @param train_ind Vector, index of the original spatial locations
#' @param test_ind Vector, index of the newly grided spatial locations
#' @param optimal_phi Numeric, the optimal phi value stored in CARD_object
#' @param ineibor Numeric, number of neighbors used in the imputation on newly
#' grided spatial locations, default is 10.
#'
#' @import Matrix
#' @importFrom RANN nn2
#' @return Return a list with the imputed Cell type composition Vtest matrix on
#' the newly grided spatial locations and predicted normalized gene expression
#'

Sigma <- function(location_orig, train_ind, test_ind, optimal_phi, ineibor) {
    norm_cords_temp <- norm_coords_train_test(
        location_orig, 
        train_ind, 
        test_ind)
    ##### find neiibors
    near_data <- nn2(norm_cords_temp[, seq_len(2)], k = ineibor + 1)
    neibors <- near_data$nn.idx
    neibors <- neibors[, -1] ##### delete the location itself as the neighbor
    nmat <- Matrix(0,
        nrow = nrow(neibors),
        ncol = nrow(neibors), 
        sparse = TRUE
    )
    
    ##### speed up the matrices when it is large
    for (icol in seq_len(ncol(neibors))) {
        edges <- data.frame(
            i = seq_len(nrow(neibors)),
            j = neibors[, icol]
        )
        adjacency <- sparseMatrix(
            i = as.integer(edges$i),
            j = as.integer(edges$j),
            x = 1,
            dims = rep(nrow(neibors), 2),
            use.last.ij = TRUE
        )
        nmat <- nmat + adjacency
    }
    
    ##### find mutual neibors, for non-mutual neighbors, the value will be zero
    nmat <- nmat * t(nmat)
    
    ##### create euclidiean distance
    dist_neibors <- near_data$nn.dists
    isigma <- 0.1
    kernel_neibors <- exp(-dist_neibors^2 / (2 * isigma^2))
    wtemp <- Matrix(0,
        nrow = nrow(neibors),
        ncol = nrow(neibors),
        sparse = TRUE
    )
    
    ##### speed up the matrices when it is large
    for (icol in seq_len(ncol(kernel_neibors))) {
        temp <- near_data$nn.idx
        edges <- data.frame(
            i = seq_len(nrow(temp)),
            j = temp[, icol]
        )
        adjacency <- sparseMatrix(
            i = as.integer(edges$i),
            j = as.integer(edges$j),
            x = kernel_neibors[, icol],
            dims = rep(nrow(neibors), 2),
            use.last.ij = TRUE
        )
        wtemp <- wtemp + adjacency
    }
    
    #### create euclidiean distance for mutual neibors, for non-mutual
    ##### neighbors, the value will be zero
    wtemp <- wtemp * nmat
    rownames(wtemp) <- rownames(norm_cords_temp)
    colnames(wtemp) <- rownames(norm_cords_temp)
    
    ##### calculate variance matrix that I need
    dtemp <- Diagonal(x = colSums(wtemp))
    In <- Diagonal(nrow(wtemp))
    w22 <- wtemp[
        (length(test_ind) + 1):nrow(wtemp),
        (length(test_ind) + 1):ncol(wtemp)
    ]
    sigmatemp <- dtemp - 
        optimal_phi * wtemp
    sigma11 <- sigmatemp[
        seq_len(length(test_ind)),
        seq_len(length(test_ind))
    ]
    sigma12 <- sigmatemp[
        seq_len(length(test_ind)),
        (length(test_ind) + 1):ncol(sigmatemp)
    ]
    sigma21 <- sigmatemp[
        (length(test_ind) + 1):nrow(sigmatemp),
        seq_len(length(test_ind))
    ]
    sigma22 <- sigmatemp[
        (length(test_ind) + 1):nrow(sigmatemp),
        (length(test_ind) + 1):ncol(sigmatemp)
    ]
    return(list(
        SigmaTemp = sigmatemp,
        Sigma11 = sigma11, 
        Sigma12 = sigma12,
        Sigma21 = sigma21, 
        Sigma22 = sigma22,
        W22 = w22
    ))
}

#' Imputation and Construction of High-Resolution Spatial Maps for Cell Type
#' Composition and Gene Expression by the spatial correlation structure between
#' original spatial locations and new grided spatial locations
#'
#' @param vtrain Matrix, estimated V matrix from CARD
#' @param location_orig Data frame, spatial location data frame of the original
#' spatial resolved transcriptomics dataset, stored in the
#' spatialCoords(CARD_object)
#' @param train_ind Vector, index of the original spatial locations
#' @param test_ind Vector, index of the newly grided spatial locations
#' @param B Matrix, used in the deconvolution as the reference basis matrix
#' @param xinput_norm Matrix, used in the deconvolution as the normalized
#' spatial count data
#' @param optimal_b Vector, vector of the intercept for each cel type estimated
#' based on the original spatial resolution
#' @param optimal_phi Numeric, the optimal phi value stored in CARD_object
#' @param lambda Vector, vector of cell type specific scalar in the CAR model
#' @param ineibor Numeric, number of neighbors used in the imputation on newly
#' grided spatial locations, default is 10.
#'
#' @return Return a list with the imputed Cell type composition Vtest matrix on
#' the newly grided spatial locations and predicted normalized gene expression
#'


mvn_cv <- function(
        vtrain, 
        location_orig, 
        train_ind, 
        test_ind, 
        B, 
        xinput_norm,
        optimal_b, 
        optimal_phi, 
        lambda, 
        ineibor) {
    ##### Calculate imputed Cell type composition Vtest matrix
    sigma_list <- Sigma(
        location_orig, 
        train_ind, 
        test_ind, 
        optimal_phi, 
        ineibor)
    sigma11 <- sigma_list$Sigma11
    sigma21 <- sigma_list$Sigma21
    sigma12 <- sigma_list$Sigma12
    sigma22 <- sigma_list$Sigma22
    w22 <- sigma_list$W22
    mu_cond <- NULL
    for (ict in seq_len(length(lambda))) {
        temp <- 
            sigma12 %*% 
            (vtrain[, ict] - optimal_b[seq_len(nrow(vtrain)), ict])
        solver <- solve(sigma11, temp)
        Mu <- optimal_b[seq_len(length(test_ind)), ict] - solver ### q * k
        mu_cond <- cbind(mu_cond, Mu)
    }
    vtest <- mu_cond
    colnames(vtest) <- colnames(vtrain)
    
    ##### Calculate the predicted normalized gene expression
    x_test_hat <- B %*% t(vtest)
    rownames(vtest) <- colnames(x_test_hat)
    vtest <- sweep(vtest, 1, rowSums(vtest), "/")
    return(list(Vtest = vtest, XtestHat = x_test_hat))
}

#' Construct an enhanced spatial expression map on the unmeasured tissue
#' locations
#'
#' @param CARD_object SpatialExperiment Object created by 
#' CARD_deconvolution with estimated cell type compositions on the
#' original spatial resolved transcriptomics data.
#' @param num_grids Initial number of newly grided spatial locations. The final
#' number of newly grided spatial locations will be lower than this value since
#' the newly grided locations outside the shape of the tissue will be filtered
#' @param ineibor Numeric, number of neighbors used in the imputation on newly
#' grided spatial locations, default is 10.
#' @param exclude Vector, the rownames of spatial location data on the original
#' resolution that you want to exclude. This is to avoid the weird detection of
#' the shape.
#'
#' @importFrom S4Vectors metadata
#' @importFrom S4Vectors metadata<-
#'
#' @return Return a SpatialExperiment object with the refined cell type 
#' compositions estimated for newly grided spots and the refined predicted 
#' gene expression (normalized).
#'
#' @export
#' @examples
#' data(spatial_count)
#' data(spatial_location)
#' data(sc_count)
#' data(sc_meta)
#' CARD_obj <- CARD_deconvolution(
#'     sc_count = sc_count,
#'     sc_meta = sc_meta,
#'     spatial_count = spatial_count,
#'     spatial_location = spatial_location,
#'     ct_varname = "cellType",
#'     ct_select = unique(sc_meta$cellType),
#'     sample_varname = "sampleInfo",
#'     mincountgene = 100,
#'     mincountspot = 5
#' )
#' CARD_obj <- CARD_imputation(
#'     CARD_obj,
#'     num_grids = 200,
#'     ineibor = 10,
#'     exclude = NULL
#' )
#'
CARD_imputation <- function(
        CARD_object, 
        num_grids, 
        ineibor = 10, 
        exclude = NULL){
    ##### Check inputs
    if (!inherits(CARD_object, "SpatialExperiment")) {
        stop("CARD_object must be a SpatialExperiment object is created by
        CARD functions")
    }
    if (!is.numeric(num_grids) || 
        length(num_grids) != 1 || 
        num_grids <= 0 || 
        num_grids != as.integer(num_grids)) {
        stop("num_grids must be a single positive integer")
    }
    if (!is.numeric(ineibor) ||
        length(ineibor) != 1 ||
        ineibor <= 0 ||
        ineibor != as.integer(ineibor)) {
        stop("ineibor must be a single positive integer")
    }
    if (!is.null(exclude) && !is.character(exclude)) {
        stop("exclude must be NULL or a character vector")
    }
    
    ##### extract results from CARD object
    B <- CARD_object@metadata$algorithm_matrix$B
    xinput_norm <- CARD_object@metadata$algorithm_matrix$Xinput_norm
    vtrain <- CARD_object@metadata$algorithm_matrix$Res$V
    location <- as.data.frame(spatialCoords(CARD_object))
    
    ##### check
    if (sum(rownames(location) == rownames(vtrain)) == nrow(vtrain)) {
        message("## The rownames of locations are matched ...\n")
    }
    
    ##### Make the new grids
    message("## Make grids on new spatial locations ...\n")
    if (!is.null(exclude)) {
        location_use_to_sample <- 
            location[!(rownames(location) %in% exclude), ]
    } else {
        location_use_to_sample <- location
    }
    data <- sample_grid_within(location_use_to_sample, num_grids, concavity = 2)
    rownames(data) <- paste0(data$x, "x", data$y)
    
    ##### delete the newly grided locations that are the same as the original
    ##### ones
    data <- data[!(rownames(data) %in% rownames(location)), ]
    location_train <- location[, seq_len(2)]
    location_test <- data[, seq_len(2)]
    location_combine <- rbind(data[, seq_len(2)], location[, seq_len(2)])
    test_ind <- seq_len(nrow(data))
    train_ind <- (nrow(data) + 1):nrow(location_combine)
    location_orig <- location_combine
    
    ##### results from the CARD
    optimal_b <- CARD_object@metadata$algorithm_matrix$Res$b
    vec_one <- rep(1, nrow(location_combine))
    optimal_b_sum <- vec_one %*% t(optimal_b)
    optimal_phi <- CARD_object@metadata$info_parameters$phi
    lambda <- CARD_object@metadata$algorithm_matrix$Res$lambda
    imputation <- 
        mvn_cv(
            vtrain, 
            location_orig, 
            train_ind, 
            test_ind,
            B, 
            xinput_norm, 
            optimal_b_sum, 
            optimal_phi, 
            lambda, 
            ineibor
        )
    spe_out <- SpatialExperiment(
        assay = list(refined_expression = as.matrix(imputation$XtestHat)))
    colData(spe_out)$refined_prop <- as.matrix(imputation$Vtest)
    
    ## remove sample_id 
    spe_out@colData@listData[["sample_id"]] <- NULL
    metadata <- metadata(CARD_object)
    metadata(spe_out) <- metadata
    metadata(spe_out)$Proportion_CARD <- CARD_object$Proportion_CARD
    metadata(spe_out)$spatial_countMat <- 
        CARD_object@assays@data$spatial_countMat
    metadata(spe_out)$spatial_location <- spatialCoords(CARD_object)
    
    return(spe_out)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' SpatialDeconv function based on Conditional Autoregressive model
#' @param XinputIn The input of normalized spatial data
#' @param UIn The input of cell type specific basis matrix B
#' @param WIn The constructed W weight matrix from Gaussian kernel
#' @param phiIn The phi value
#' @param max_iterIn Maximum iterations
#' @param epsilonIn epsilon for convergence
#' @param initV Initial matrix of cell type compositions V
#' @param initb Initial vector of cell type specific intercept
#' @param initSigma_e2 Initial value of residual variance
#' @param initLambda Initial vector of cell type sepcific scalar.
#'
#' @return A list
CARDfree <- function(XinputIn, UIn, WIn, phiIn, max_iterIn, epsilonIn, initV, initb, initSigma_e2, initLambda) {
    .Call('_CARDspa_CARDfree', PACKAGE = 'CARDspa', XinputIn, UIn, WIn, phiIn, max_iterIn, epsilonIn, initV, initb, initSigma_e2, initLambda)
}

#' SpatialDeconv function based on Conditional Autoregressive model
#' @param XinputIn The input of normalized spatial data
#' @param UIn The input of cell type specific basis matrix B
#' @param WIn The constructed W weight matrix from Gaussian kernel
#' @param phiIn The phi value
#' @param max_iterIn Maximum iterations
#' @param epsilonIn epsilon for convergence
#' @param initV Initial matrix of cell type compositions V
#' @param initb Initial vector of cell type specific intercept
#' @param initSigma_e2 Initial value of residual variance
#' @param initLambda Initial vector of cell type sepcific scalar.
#'
#' @return A list
CARDref <- function(XinputIn, UIn, WIn, phiIn, max_iterIn, epsilonIn, initV, initb, initSigma_e2, initLambda) {
    .Call('_CARDspa_CARDref', PACKAGE = 'CARDspa', XinputIn, UIn, WIn, phiIn, max_iterIn, epsilonIn, initV, initb, initSigma_e2, initLambda)
}

# setup-card.R
data(spatial_count)
data(spatial_location)
data(sc_count)
data(sc_meta)


CARD_obj <- CARD_deconvolution(
    sc_count = sc_count,
    sc_meta = sc_meta,
    spatial_count = spatial_count,
    spatial_location = spatial_location,
    ct_varname = "cellType",
    ct_select = unique(sc_meta$cellType),
    sample_varname = "sampleInfo",
    mincountgene = 10,
    mincountspot = 5
)
assign("CARD_obj", CARD_obj, envir = .GlobalEnv)

#' Spatial location data
#'
#' The spatial location data must be in the format of data frame while each row
#' represents a spatial location, the first column represents the x coordinate
#' and the second column represents the y coordinate. The rownames of the
#' spatial location data frame should match exactly with the column names of the
#' spatial_count.
#'
#'
#' @docType data
#' @keywords datasets
#' @usage data(spatial_location)
#' @name spatial_location
"spatial_location"

---
title: "Example_Analysis"
author: "Ying Ma"
date: "Jun 25, 2024"
output:
    BiocStyle::html_document:
        toc: true
        toc_depth: 2
vignette: >
    %\VignetteIndexEntry{Example_Analysis}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
```

## Introduction
We developed a statistical method for spatially informed cell type 
deconvolution for spatial transcriptomics. Briefly,CARD is a 
reference-based deconvolution method that estimates cell type composition 
in spatial transcriptomics based on cell type specific expression 
information obtained from a reference scRNA-seq data. A key feature of 
CARD is its ability to accommodate spatial correlation in the cell type 
composition across tissue locations, enabling accurate and spatially 
informed cell type deconvolution as well as refined spatial map 
construction. CARD relies on an efficient optimization algorithm for 
constrained maximum likelihood estimation and is scalable to spatial 
transcriptomics with tens of thousands of spatial locations 
and tens of thousands of genes.

## Installation
```{r, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE)) {
    install.packages("BiocManager")
}
BiocManager::install("CARDspa")
```

This tutorial is the example analysis with CARDspa on the human pancreatic 
ductal adenocarcinomas data from Moncada et al, 2020. Please note that we are 
using edited data, see the [tutorial](https://yma-lab.github.io/CARD/) for an 
example using the complete data.

## Required input data
`CARD` requires two types of input data:
- spatial transcriptomics count data, along with spatial location information.  
- single cell RNAseq (scRNA-seq) count data, along with meta information 
indicating the cell type information and the sample (subject) information for 
each cell.  

The example data for running the tutorial is included in the package.
Here are the details about the required data input illustrated by the example 
datasets. 

### 1. spatial transcriptomics data, e.g.,
```{r}
library(CARDspa)
library(RcppML)
library(NMF)
library(RcppArmadillo)
library(SingleCellExperiment)
library(SpatialExperiment)
library(ggplot2)
#### load the example spatial transcriptomics count data,
data(spatial_count)
spatial_count[1:4, 1:4]
```
The spatial transcriptomics count data must be in the format of matrix or 
sparseMatrix, while each row represents a gene and each column represents a 
spatial location. The column names of the spatial data can be in the 
"XcoordxYcoord" (i.e., 10x10) format, but you can also maintain your original 
spot names, for example, barcode names. 

```{r}
#### load the example spatial location data,
data(spatial_location)
spatial_location[1:4, ]
```
The spatial location data must be in the format of data frame while each row 
represents a spatial location, the first column represents the x coordinate and 
the second column represents the y coordinate. The rownames of the spatial 
location data frame should match exactly with the column names of the 
spatial_count.

### 2. single cell RNAseq ((scRNA-seq)) data,  e.g.,
```{r}
data(sc_count)
sc_count[1:4, 1:4]
```
The scRNA-seq count data must be in the format of matrix or sparseMatrix, while 
each row represents a gene and each column represents a cell.

```{r}
data(sc_meta)
sc_meta[1:4, ]
```

The scRNAseq meta data must be in the format of data frame while each row 
represents a cell. The rownames of the scRNAseq meta data should match exactly 
with the column names of the scRNAseq count data. The sc_meta data must contain 
the column indicating the cell type assignment for each cell (e.g., "cellType" 
column in the example sc_meta data). Sample/subject information should be 
provided, if there is only one sample, we can add a column by 
```sc_meta$sampleInfo = "sample1"```.

We suggest the users to check their single cell RNASeq data carefully before 
running CARD. We suggest the users to input the single cell RNAseq data with 
each cell type containing at least 2 cells. i.e. 
print(table(sc_meta$cellType,useNA = "ifany"))

## Cell Type Deconvolution
### 1. Deconvolution using CARD
We can use `CARD_deconvolution` to deconvolute the spatial transcriptomics
data. The essential inputs are:
- sc_count: Matrix or sparse matrix of raw scRNA-seq count data, each row 
represents a gene and each column represents a cell. This sc_count data 
serves as a reference for the cell type deconvolution for spatial 
transcriptomics data. 
- sc_meta: Data frame, with each row representing the cell type and/or sample 
information of a specific cell. The row names of this data frame should match 
exactly with the column names of the sc_count data. The sc_meta data must 
contain the column indicating the cell type assignment for each cell 
(e.g., "cellType" column in the example sc_meta data). 
- spatial_count: Matrix or sparse matrix of raw spatial resolved 
transcriptomics count data, each row represents a gene and each column 
represents a spatial location. 
This is the spatial transcriptomics data that we are interested to deconvolute.
- spatial_location: Data frame, with two columns representing the x and y 
coordinates of the spatial location. The rownames of this data frame should 
match eaxctly with the columns of the spatial_count.
- ct.varname: Caracter, the name of the column in sc_meta that specifies the
cell type assignment. 
- ct.select: Vector of cell type names that you are interested in to 
deconvolute, default as NULL. If NULL, then use all cell types provided by 
single cell dataset.
- sample.varname: Character,the name of the column in sc_meta that
specifies the sample/subject information. If NULL, we just use the 
whole data as one sample/subject.
- minCountGene: Numeric, include spatial locations where at least this
number of counts detected. Default is 100. 
- minCountSpot: Numeric, include genes where at least this number of spatial
locations that have non-zero expression. Default is 5.

This function first create a CARD object and then do deconvolution. Finally 
return a SpatialExperiment object. The results are stored in 
`CARD_obj$Proportion_CARD`


CARD is computationally fast and memory 
efficient. CARD relies on an efficient optimization algorithm for constrained 
maximum likelihood estimation and is scalable to spatial transcriptomics with 
tens of thousands of spatial locations and tens of thousands of genes. 
For the example dataset with the sample size of 428 locations, it takes 
within a minute to finish the deconvolution. 

```{r}
set.seed(seed = 20200107)
CARD_obj <- CARD_deconvolution(
    sc_count = sc_count,
    sc_meta = sc_meta,
    spatial_count = spatial_count,
    spatial_location = spatial_location,
    ct_varname = "cellType",
    ct_select = unique(sc_meta$cellType),
    sample_varname = "sampleInfo",
    mincountgene = 100,
    mincountspot = 5 )
## QC on scRNASeq dataset! ...
## QC on spatially-resolved dataset! ..
## create reference matrix from scRNASeq...
## Select Informative Genes! ...
## Deconvolution Starts! ...
## Deconvolution Finish! ...
```

And `CARDspa` also supports using SingleCellExperiment object and 
SingleCellExperiment object, you can run the following code:

```{r}
## create sce object
sce <- SingleCellExperiment(assay = list(counts = sc_count),
                            colData = sc_meta)
## create spe object
spe <- SpatialExperiment(assay = list(counts = spatial_count),
                        spatialCoords  = as.matrix(spatial_location)
                        )
celltypes <- unique(sc_meta$cellType)
    
set.seed(seed = 20200107)
CARD_obj <- CARD_deconvolution(
    spe = spe,
    sce = sce,
    sc_count = NULL,
    sc_meta = NULL,
    spatial_count = NULL,
    spatial_location = NULL,
    ct_varname = "cellType",
    ct_select = celltypes,
    sample_varname = "sampleInfo",
    mincountgene = 100,
    mincountspot = 5
)
```

The spatial data are stored in `assays(CARD_obj)$spatial_countMat` and 
`spatialCoords(CARD_obj)` while the scRNA-seq data is stored in 
`CARD_obj@metadata$sc_eset` in the format of SingleCellExperiment. 
The results are stored in `CARD_obj$Proportion_CARD`.

```{r}
print(CARD_obj$Proportion_CARD[1:2, ])
```

### 2. Visualize the proportion for each cell type
First, we jointly visualize the cell type proportion matrix through scatterpie 
plot.Note that here because the number of spots is relatively small, so jointly 
visualize the cell type proportion matrix in the scatterpie plot format is 
duable. We do not recommend users to visualize this plot when the number of 
spots is > 500. Instead, we recommend users to visualize the proportion 
directly, i.e., using the function CARD_visualize_prop(). Details of using this 
function see the next example.  

```{r}
## set the colors. Here, I just use the colors in the manuscript, if the color
## is not provided, the function will use default color in the package.
colors <- c(
    "#FFD92F", "#4DAF4A", "#FCCDE5", "#D9D9D9", "#377EB8", "#7FC97F",
    "#BEAED4", "#FDC086", "#FFFF99", "#386CB0", "#F0027F", "#BF5B17",
    "#666666", "#1B9E77", "#D95F02", "#7570B3", "#E7298A", "#66A61E",
    "#E6AB02", "#A6761D"
)
p1 <- CARD_visualize_pie(
    proportion = CARD_obj$Proportion_CARD,
    spatial_location = spatialCoords(CARD_obj),
    colors = colors,
    radius = 0.52
) ### You can choose radius = NULL or your own radius number
print(p1)
```

Then, we can select some interested cell types to visualize separately. 

```{r}
## select the cell type that we are interested
ct.visualize <- c(
    "Acinar_cells", "Cancer_clone_A", "Cancer_clone_B",
    "Ductal_terminal_ductal_like",
    "Ductal_CRISP3_high-centroacinar_like",
    "Ductal_MHC_Class_II",
    "Ductal_APOL1_high-hypoxic",
    "Fibroblasts"
)
## visualize the spatial distribution of the cell type proportion
p2 <- CARD_visualize_prop(
    proportion = CARD_obj$Proportion_CARD,
    spatial_location = spatialCoords(CARD_obj),
    ### selected cell types to visualize
    ct_visualize = ct.visualize,
    ### if not provide, we will use the default colors
    colors = c("lightblue", "lightyellow", "red"),
    ### number of columns in the figure panel
    NumCols = 4,
    ### point size in ggplot2 scatterplot
    pointSize = 3.0
)
print(p2)
```

### 3. Visualize the proportion for two cell types
We added a new visualization function to visualize the distribution of two cell 
types on the same post.  

```{r}
## visualize the spatial distribution of two cell types on the same plot
p3 <- CARD_visualize_prop_2CT(
    ### Cell type proportion estimated by CARD
    proportion = CARD_obj$Proportion_CARD,
    ### spatial location information
    spatial_location = spatialCoords(CARD_obj),
    ### two cell types you want to visualize
    ct2_visualize = c("Cancer_clone_A", "Cancer_clone_B"),
    ### two color scales
    colors = list(
        c("lightblue", "lightyellow", "red"),
        c("lightblue", "lightyellow", "black")
    )
)
print(p3)
```

### 5. Visualize the cell type proportion correlation 
```{r}
# if not provide, we will use the default colors
p4 <- CARD_visualize_Cor(CARD_obj$Proportion_CARD, colors = NULL)
print(p4)
```

## Refined spatial map
A unique feature of CARD is its ability to model the spatial 
correlation in cell type composition across tissue locations, thus 
enabling spatially informed cell type deconvolution. Modeling spatial 
correlation allows us to not only accurately infer the cell type 
composition on each spatial location, but also impute cell type 
compositions and gene expression levels on unmeasured tissue locations, 
facilitating the construction of a refined spatial tissue map with a 
resolution much higher than that measured in the original study.
Specifically, CARD constructed a refined spatial map through the 
function `CARD_imputation`. The essential inputs are:

- CARD_object: SpatialExperiment Object created by CARD_deconvolution with 
estimated cell type compositions on the original spatial resolved 
transcriptomics data.
- NumGrids: Initial number of newly grided spatial locations. The final number 
of newly grided spatial locations will be lower than this value since the newly 
grided locations outside the shape of the tissue will be filtered. 
- ineibor: Numeric, number of neighbors used in the imputation on newly grided 
spatial locations, default is 10.  

Briefly, CARD first outlined the shape of the tissue by applying a 
two-dimensional concave hull algorithm on the existing locations, then perform 
imputation on the newly grided spatial locations. We recommend to check the 
exisiting spatial locations to see if there are outliers that are seriously 
affect the detection of the shape. 

### 1. Imputation on the newly grided spatial locations

```{r}
CARD_obj <- CARD_imputation(
    CARD_obj, 
    num_grids = 2000, 
    ineibor = 10, 
    exclude = NULL)
## The rownames of locations are matched ...
## Make grids on new spatial locations ...
```

The results are store in `CARD_obj$refined_prop` and 
`assays(CARD_obj)$refined_expression`

```{r}
## Visualize the newly grided spatial locations to see if the shape is correctly
## detected. If not, the user can provide the row names of the excluded spatial
## location data into the CARD_imputation function
location_imputation <- cbind.data.frame(
    x = as.numeric(sapply(
        strsplit(rownames(CARD_obj$refined_prop), split = "x"), "[", 1
    )),
    y = as.numeric(sapply(
        strsplit(rownames(CARD_obj$refined_prop), split = "x"), "[", 2
    ))
)
rownames(location_imputation) <- rownames(CARD_obj$refined_prop)
library(ggplot2)
p5 <- ggplot(
    location_imputation,
    aes(x = x, y = y)
) +
    geom_point(shape = 22, color = "#7dc7f5") +
    theme(
        plot.margin = margin(0.1, 0.1, 0.1, 0.1, "cm"),
        legend.position = "bottom",
        panel.background = element_blank(),
        plot.background = element_blank(),
        panel.border = element_rect(colour = "grey89", fill = NA, 
                                    linewidth = 0.5)
    )
print(p5)
```


### 2. Visualize the cell type proportion at an enhanced resolution
Now we can use the same `CARD_visualize_prop` function to visualize the cell 
type proportion at the enhanced resolution. But this time, the input of the 
function should be the imputed cell typr propotion and corresponding newly 
grided spatial locations.

```{r}
p6 <- CARD_visualize_prop(
    proportion = CARD_obj$refined_prop,
    spatial_location = location_imputation,
    ct_visualize = ct.visualize,
    colors = c("lightblue", "lightyellow", "red"),
    NumCols = 4
)
print(p6)
```

### 3. Visualize the marker gene expression at an enhanced resolution
After we obtained cell type proportion at the enhanced resolution by CARD, we 
can predict the spatial gene expression at the enhanced resolution. The 
following code is to visualize the marker gene expression at an enhanced 
resolution.

```{r}
p7 <- CARD_visualize_gene(
    spatial_expression = assays(CARD_obj)$refined_expression,
    spatial_location = location_imputation,
    gene_visualize = c("A4GNT", "AAMDC", "CD248"),
    colors = NULL,
    NumCols = 6
)
print(p7)
```

Now, compare with the original resolution, CARD facilitates the construction of 
a refined spatial tissue map with a resolution much higher than that 
measured in the original study.

```{r}
p8 <- CARD_visualize_gene(
    spatial_expression = metadata(CARD_obj)$spatial_countMat,
    spatial_location = metadata(CARD_obj)$spatial_location,
    gene_visualize = c("A4GNT", "AAMDC", "CD248"),
    colors = NULL,
    NumCols = 6
)
print(p8)
```


## Extension of CARD in a reference-free version: CARDfree
We extended CARD to enable reference-free cell type deconvolution and eliminate 
the need for the single-cell reference data. We refer to this extension as the 
reference-free version of CARD, or simply as CARDfree. Different from CARD, 
CARDfree no longer requires an external single-cell reference dataset and only 
needs users to input a list of gene names for previously known cell type markers
We use the same exmple dataset to illustrate the use of CARDfree. 
In addition to the exmple dataset, CARDfree also requires the input of 
marker gene list, which is in a list format with each element of the list 
being the cell type specific gene markers. The example marker list for runing 
the tutorial is included. 

Similar to CARD, we will first need to create a CARDfree object with 
the spatial transcriptomics dataset and the marker gene list 


### 1. Deconvolution using CARDfree
We can use `CARD_refFree` to do reference-free deconvolution. This function 
frist creat a CARDfree object and then do deconvolution. Briefly, 
the essential inputs are the same as the function `CARD_deconvolution`, except 
that this function does not require the single cell count and meta information 
matrix. Instead, it requires a markerList. 

```{r}
## deconvolution using CARDfree
data(markerList)
set.seed(seed = 20200107)
CARDfree_obj <- CARD_refFree(
    markerlist = markerList,
    spatial_count = spatial_count,
    spatial_location = spatial_location,
    mincountgene = 100,
    mincountspot = 5
)
```

Similarly, you can use SpatialExperiment object.

```{r}
data(markerList)
set.seed(seed = 20200107)
CARDfree_obj <- CARD_refFree(
    markerlist = markerList,
    spatial_count = NULL,
    spatial_location = NULL,
    spe = spe,
    mincountgene = 100,
    mincountspot = 5
)
```


The spatial data are stored in `assays(CARDfree_obj)$spatial_countMat` 
and `spatialCoords(CARDfree_obj)` while the marker list is stored in 
`CARDfree_obj@metadata$markerList` in the format of list. 
The results are stored in `CARDfree_obj$Proportion_CARD`. 

```{r}
## One limitation of reference-free version of CARD is that the cell
## types inferred
## from CARDfree do not come with a cell type label. It might be difficult to
## interpret the results.
print(CARDfree_obj$Proportion_CARD[1:2, ])
```

### 3. Visualization of the results of CARDfree
Note that here because the number of spots is relatively small, so jointly 
visualize the cell type proportion matrix in the scatterpie plot format is 
duable. We do not recommend users to visualize this plot when the number of 
spots is > 500. Instead, we recommend users to visualize the proportion 
directly, i.e., using the function CARD_visualize_prop(). 

```{r}
colors <- c(
    "#FFD92F", "#4DAF4A", "#FCCDE5", "#D9D9D9", "#377EB8", "#7FC97F", "#BEAED4",
    "#FDC086", "#FFFF99", "#386CB0", "#F0027F", "#BF5B17", "#666666",
    "#1B9E77", "#D95F02", "#7570B3", "#E7298A", "#66A61E", "#E6AB02",
    "#A6761D"
)
### In order to maximumply match with the original results of CARD, we order the
### colors to generally match with the results infered by CARD
current_data <- CARDfree_obj$Proportion_CARD
new_order <- current_data[, c(
    8, 10, 14, 2, 1, 6, 12, 18, 7, 13, 20, 19, 16,
    17, 11, 15, 4, 9, 3, 5
)]
CARDfree_obj$Proportion_CARD <- new_order
colnames(CARDfree_obj$Proportion_CARD) <- paste0("CT", 1:20)
p9 <- CARD_visualize_pie(CARDfree_obj$Proportion_CARD,
    spatialCoords(CARDfree_obj),
    colors = colors
)
print(p9)
```

## Extension of CARD for single cell resolution mapping
We also extended CARD to facilitate the construction of single-cell resolution 
spatial transcriptomics from non-single-cell resolution spatial 
transcriptomics. 
Details of the algorithm see the main text. Briefly, we infer the single cell 
resolution gene expression for each measured spatial location from the 
non-single cell resolution spatial transcriptomics data based on reference 
scRNaseq data we used for deconvolution. The procedure is implemented in the 
function `CARD_SCMapping`. 
The essential inputs are:
- CARD_object: CARD object create by the createCARDObject function. This one 
should be the one after we finish the deconvolution procedure
- shapeSpot: a character indicating whether the sampled spatial coordinates for 
single cells locating in a Square-like region or a Circle-like region. The 
center of this region is the measured spatial location in the non-single cell 
resolution spatial transcriptomics data. The default is "Square", and the other 
option is "Circle"
- numCell: a numeric value indicating the number of cores used to accelerate 
the procedure.
```{r}
#### Note that here the shapeSpot is the user defined variable which
#### indicates the capturing area of single cells. Details see above.
set.seed(seed = 20210107)
scMapping <- CARD_scmapping(CARD_obj, shapeSpot = "Square", numcell = 20, 
                            ncore = 2)
print(scMapping)
### spatial location info and expression count of the single cell resolution
### data
MapCellCords <- as.data.frame(colData(scMapping))
count_SC <- assays(scMapping)$counts
```

The results are stored in a SingleCellExperiment object with mapped single cell 
resolution counts stored in the assays slot and the information of the spatial 
location for each single cell as well as their relashionship to the original 
measured spatial location is stored in the colData slot. 

Next, we visualize the cell type for each single cell with their spatial 
location information 
```{r}
df <- MapCellCords
colors <- c(
    "#8DD3C7", "#CFECBB", "#F4F4B9", "#CFCCCF", "#D1A7B9", "#E9D3DE", "#F4867C",
    "#C0979F", "#D5CFD6", "#86B1CD", "#CEB28B", "#EDBC63", "#C59CC5",
    "#C09CBF", "#C2D567", "#C9DAC3", "#E1EBA0",
    "#FFED6F", "#CDD796", "#F8CDDE"
)
p10 <- ggplot(df, aes(x = x, y = y, colour = CT)) +
    geom_point(size = 3.0) +
    scale_colour_manual(values = colors) +
    # facet_wrap(~Method,ncol = 2,nrow = 3) +
    theme(
        plot.margin = margin(0.1, 0.1, 0.1, 0.1, "cm"),
        panel.background = element_rect(colour = "white", fill = "white"),
        plot.background = element_rect(colour = "white", fill = "white"),
        legend.position = "bottom",
        panel.border = element_rect(
            colour = "grey89", 
            fill = NA, 
            linewidth = 0.5),
        axis.text = element_blank(),
        axis.ticks = element_blank(),
        axis.title = element_blank(),
        legend.title = element_text(size = 13, face = "bold"),
        legend.text = element_text(size = 12),
        legend.key = element_rect(colour = "transparent", fill = "white"),
        legend.key.size = unit(0.45, "cm"),
        strip.text = element_text(size = 15, face = "bold")
    ) +
    guides(color = guide_legend(title = "Cell Type"))
print(p10)
```

## Session information
```{r}
sessionInfo()
```

## References
Ma, Y., Zhou, X. Spatially informed cell-type deconvolution for spatial 
transcriptomics. Nat Biotechnol 40, 1349–1359 (2022).
https://doi.org/10.1038/s41587-022-01273-7

Moncada, R., Barkley, D., Wagner, F. et al. Integrating 
microarray-based spatial transcriptomics and single-cell RNA-seq 
reveals tissue architecture in pancreatic ductal adenocarcinomas. 
Nat Biotechnol 38, 333–342 (2020).
https://doi.org/10.1038/s41587-019-0392-8

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CARD.imputation.R
\name{norm_coords_train_test}
\alias{norm_coords_train_test}
\title{Normalize the new spatial locations without changing the shape and
relative positions}
\usage{
norm_coords_train_test(location_orig, train_ind, test_ind)
}
\arguments{
\item{location_orig}{Data frame, spatial location data frame of the original
spatial resolved transcriptomics dataset, stored in the
spatialCoords(CARD_object)}

\item{train_ind}{Vector, Index of the original spatial locations}

\item{test_ind}{Vector, Index of the newly grided spatial locations}
}
\value{
Return the normalized spatial location data frame
}
\description{
Normalize the new spatial locations without changing the shape and
relative positions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{CARD_visualize_pie}
\alias{CARD_visualize_pie}
\title{Visualize the spatial distribution of cell type proportion in a geom
scatterpie plot}
\usage{
CARD_visualize_pie(proportion, spatial_location, colors = NULL, radius = NULL)
}
\arguments{
\item{proportion}{Data frame, cell type proportion estimated by CARD in
either original resolution or enhanced resolution.}

\item{spatial_location}{Data frame, spatial location information.}

\item{colors}{Vector of color names that you want to use, if NULL, we will
use the color palette "Spectral" from RColorBrewer package.}

\item{radius}{Numeric value about the radius of each pie chart, if NULL, we
will calculate it inside the function.}
}
\value{
Returns a ggplot2 figure.
}
\description{
Visualize the spatial distribution of cell type proportion in a geom
scatterpie plot
}
\examples{
library(ggplot2)
library(SpatialExperiment)
data(spatial_count)
data(spatial_location)
data(sc_count)
data(sc_meta)
CARD_obj <- CARD_deconvolution(
    sc_count = sc_count,
    sc_meta = sc_meta,
    spatial_count = spatial_count,
    spatial_location = spatial_location,
    ct_varname = "cellType",
    ct_select = unique(sc_meta$cellType),
    sample_varname = "sampleInfo",
    mincountgene = 100,
    mincountspot = 5
)
colors <- c(
    "#FFD92F", "#4DAF4A", "#FCCDE5", "#D9D9D9", "#377EB8", "#7FC97F",
    "#BEAED4", "#FDC086", "#FFFF99", "#386CB0", "#F0027F", "#BF5B17", 
    "#666666", "#1B9E77", "#D95F02", "#7570B3", "#E7298A", "#66A61E", 
    "#E6AB02", "#A6761D"
)
CARD_visualize_pie(
    proportion = CARD_obj$Proportion_CARD,
    spatial_location = spatialCoords(CARD_obj),
    colors = colors,
    radius = 0.52
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CARD.imputation.R
\name{Sigma}
\alias{Sigma}
\title{Calculate the variance covariance matrix used in the imputation of the
new grided locations}
\usage{
Sigma(location_orig, train_ind, test_ind, optimal_phi, ineibor)
}
\arguments{
\item{location_orig}{Data frame, spatial location data frame of the original
spatial resolved transcriptomics dataset, stored in the
spatialCoords(CARD_object)}

\item{train_ind}{Vector, index of the original spatial locations}

\item{test_ind}{Vector, index of the newly grided spatial locations}

\item{optimal_phi}{Numeric, the optimal phi value stored in CARD_object}

\item{ineibor}{Numeric, number of neighbors used in the imputation on newly
grided spatial locations, default is 10.}
}
\value{
Return a list with the imputed Cell type composition Vtest matrix on
the newly grided spatial locations and predicted normalized gene expression
}
\description{
Calculate the variance covariance matrix used in the imputation of the
new grided locations
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatial_count.R
\docType{data}
\name{spatial_count}
\alias{spatial_count}
\title{Spatial transcriptomics count data}
\format{
An object of class \code{dgCMatrix} with 11000 rows and 428 columns.
}
\usage{
data(spatial_count)
}
\description{
The spatial transcriptomics count data must be in the format of matrix or
sparseMatrix, while each row represents a gene and each column represents a
spatial location. The column names of the spatial data can be in the
“XcoordxYcoord” (i.e., 10x10) format, but you can also maintain your original
spot names, for example, barcode names.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatial_location.R
\docType{data}
\name{spatial_location}
\alias{spatial_location}
\title{Spatial location data}
\format{
An object of class \code{data.frame} with 428 rows and 2 columns.
}
\usage{
data(spatial_location)
}
\description{
The spatial location data must be in the format of data frame while each row
represents a spatial location, the first column represents the x coordinate
and the second column represents the y coordinate. The rownames of the
spatial location data frame should match exactly with the column names of the
spatial_count.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{CARD_visualize_gene}
\alias{CARD_visualize_gene}
\title{Visualize the spatial distribution of cell type proportion}
\usage{
CARD_visualize_gene(
  spatial_expression,
  spatial_location,
  gene_visualize,
  colors = colors,
  NumCols
)
}
\arguments{
\item{spatial_expression}{Data frame, spatial gene expression in either
original resolution or enhanced resolution.}

\item{spatial_location}{Data frame, spatial location information.}

\item{gene_visualize}{Vector of selected gene names that are interested to
visualize}

\item{colors}{Vector of color names that you want to use, if NULL, we will
use the default color scale in virdis palette}

\item{NumCols}{Numeric, number of columns in the figure panel, it depends on
the number of cell types you want to visualize.}
}
\value{
Returns a ggplot2 figure.
}
\description{
Visualize the spatial distribution of cell type proportion
}
\examples{
library(ggplot2)
library(SummarizedExperiment)
library(SpatialExperiment)
data(spatial_count)
data(spatial_location)
data(sc_count)
data(sc_meta)
CARD_obj <- CARD_deconvolution(
    sc_count = sc_count,
    sc_meta = sc_meta,
    spatial_count = spatial_count,
    spatial_location = spatial_location,
    ct_varname = "cellType",
    ct_select = unique(sc_meta$cellType),
    sample_varname = "sampleInfo",
    mincountgene = 100,
    mincountspot = 5
)
CARD_visualize_gene(
    spatial_expression = assays(CARD_obj)$spatial_countMat,
    spatial_location = spatialCoords(CARD_obj),
    gene_visualize = c("A4GNT", "AAMDC", "CD248"),
    colors = NULL,
    NumCols = 3
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{CARD_visualize_Cor}
\alias{CARD_visualize_Cor}
\title{Visualize the cell type proportion correlation}
\usage{
CARD_visualize_Cor(proportion, colors = colors)
}
\arguments{
\item{proportion}{Data frame, cell type proportion estimated by CARD in
either original resolution or enhanced resolution.}

\item{colors}{Vector of color names that you want to use, if NULL, we will
use the default color scale c("#91a28c","white","#8f2c37")}
}
\value{
Returns a ggcorrplot figure.
}
\description{
Visualize the cell type proportion correlation
}
\examples{
library(ggplot2)
data(spatial_count)
data(spatial_location)
data(sc_count)
data(sc_meta)
CARD_obj <- CARD_deconvolution(
    sc_count = sc_count,
    sc_meta = sc_meta,
    spatial_count = spatial_count,
    spatial_location = spatial_location,
    ct_varname = "cellType",
    ct_select = unique(sc_meta$cellType),
    sample_varname = "sampleInfo",
    mincountgene = 100,
    mincountspot = 5
)
CARD_visualize_Cor(CARD_obj$Proportion_CARD, colors = NULL)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sc_count.R
\docType{data}
\name{sc_count}
\alias{sc_count}
\title{scRNA-seq count data}
\format{
An object of class \code{dgCMatrix} with 7000 rows and 1926 columns.
}
\usage{
data(sc_count)
}
\description{
The scRNA-seq count data must be in the format of matrix or sparseMatrix,
while each row represents a gene and each column represents a cell.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{CARD_visualize_prop}
\alias{CARD_visualize_prop}
\title{Visualize the spatial distribution of cell type proportion}
\usage{
CARD_visualize_prop(
  proportion,
  spatial_location,
  ct_visualize = ct_visualize,
  colors = c("lightblue", "lightyellow", "red"),
  NumCols,
  pointSize = 3
)
}
\arguments{
\item{proportion}{Data frame, cell type proportion estimated by CARD in
either original resolution or enhanced resolution.}

\item{spatial_location}{Data frame, spatial location information.}

\item{ct_visualize}{Vector of selected cell type names that are
interested to visualize}

\item{colors}{Vector of color names that you want to use, if NULL, we will
use the default color scale c("lightblue","lightyellow","red")}

\item{NumCols}{Numeric, number of columns in the figure panel, it depends on
the number of cell types you want to visualize.}

\item{pointSize}{Size of each point used for plotting}
}
\value{
Returns a ggplot2 figure.
}
\description{
Visualize the spatial distribution of cell type proportion
}
\examples{
library(ggplot2)
library(SpatialExperiment)
data(spatial_count)
data(spatial_location)
data(sc_count)
data(sc_meta)
CARD_obj <- CARD_deconvolution(
    sc_count = sc_count,
    sc_meta = sc_meta,
    spatial_count = spatial_count,
    spatial_location = spatial_location,
    ct_varname = "cellType",
    ct_select = unique(sc_meta$cellType),
    sample_varname = "sampleInfo",
    mincountgene = 100,
    mincountspot = 5
)
ct_visualize <- c(
    "Acinar_cells", "Cancer_clone_A", "Cancer_clone_B",
    "Ductal_terminal_ductal_like", "Ductal_CRISP3_high-centroacinar_like",
    "Ductal_MHC_Class_II", "Ductal_APOL1_high-hypoxic", "Fibroblasts"
)
CARD_visualize_prop(
    proportion = CARD_obj$Proportion_CARD,
    spatial_location = spatialCoords(CARD_obj),
    ct_visualize = ct_visualize,
    colors = c("lightblue", "lightyellow", "red"),
    NumCols = 4,
    pointSize = 3.0
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{show,CARDfree-method}
\alias{show,CARDfree-method}
\title{Show method for the CARDfree class}
\usage{
\S4method{show}{CARDfree}(object)
}
\arguments{
\item{object}{An object of class \code{CARDfree}.}
}
\value{
A concise summary of the \code{CARDfree} object is
printed to the console.
}
\description{
This method provides a concise summary of an object of
class \code{CARDfree}, displaying key information including the project
name, the number of spots, the number of cell types, and a sample of the
\code{Proportion_CARD} matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CARD.SCMapping.R
\name{assign_sc_cords}
\alias{assign_sc_cords}
\title{The function to assign the spatial location information for
each single cell}
\usage{
assign_sc_cords(mappint_spot_cell_cor, cords_new, numcell, sc_eset, ct_varname)
}
\arguments{
\item{mappint_spot_cell_cor}{a mapped correlation matrix indicating the
relashionship between each measured spatial location and the single cell in
the scRNAseq reference}

\item{cords_new}{output from the function get_high_res_cords}

\item{numcell}{a numeric value indicating the number of single cells in each
measured location, we suggest 20 for ST technology, 7 for 10x Viisum and 2
for Slide-seq}

\item{sc_eset}{a single cell experiment object stored in CARD object}

\item{ct_varname}{character, the name of the column in metaData that
specifies the cell type annotation information, stroed in CARD object}
}
\value{
Return the assigned spatial location information for the mapped
single cell
}
\description{
The function to assign the spatial location information for
each single cell
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{sc_QC}
\alias{sc_QC}
\title{Quality control of scRNA-seq count data}
\usage{
sc_QC(
  counts_in,
  metadata,
  ct_varname,
  ct_select,
  sample_varname = NULL,
  min_cells = 0,
  min_genes = 0
)
}
\arguments{
\item{counts_in}{Raw scRNAseq count data, each column is a cell and each
row is a gene.}

\item{metadata}{data frame, metadata with "ct_varname" specify the cell type
annotation information and "sample_varname" specify the sample information}

\item{ct_varname}{character, the name of the column in metadata that
specifies the cell type annotation information}

\item{ct_select}{vector of cell type names that you are interested in to
deconvolute, default as NULL. If NULL, then use all cell types provided by
single cell dataset;}

\item{sample_varname}{character,the name of the column in metadata that
specifies the sample information. If NULL, we just use the whole as one
sample.}

\item{min_cells}{numeric, we filtered out the non-expressed cells.}

\item{min_genes}{numeric we filtered out the non-expressed genes}
}
\value{
Return the filtered scRNA-seq data and meta data stored in a S4 class
(SingleCellExperiment)
}
\description{
Quality control of scRNA-seq count data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CARD.refFree.R
\name{CARD_refFree}
\alias{CARD_refFree}
\title{Extension of CARD into a reference-free version of
deconvolution: CARDfree.}
\usage{
CARD_refFree(
  markerlist,
  spatial_count,
  spatial_location,
  mincountgene = 100,
  mincountspot = 5,
  spe = NULL
)
}
\arguments{
\item{markerlist}{a list of marker genes, with each element of the list
being the vector of cell type specific marker genes}

\item{spatial_count}{Raw spatial resolved transcriptomics data, each column
is a spatial location, and each row is a gene.}

\item{spatial_location}{data frame, with two columns representing the x and
y coordinates of the spatial location. The rownames of this data frame
should match eaxctly with the columns of the spatial_count.}

\item{mincountgene}{Minimum counts for each gene}

\item{mincountspot}{Minimum counts for each spatial location}

\item{spe}{a \code{SpatialExperiment} object containing spatial data in
the \code{counts} assay, and spatial coordinates in the spatialCoords.}
}
\value{
Returns a SpatialExperiment object with estimated cell type
proportion stored in object$Proportion_CARD. Because this is a
reference-free version, the columns of estimated proportion is not cell
type but cell type cluster
}
\description{
Extension of CARD into a reference-free version of
deconvolution: CARDfree.
}
\examples{
library(RcppML)
library(NMF)
library(RcppArmadillo)
data(markerList)
data(spatial_count)
data(spatial_location)
CARDfree_obj <- CARD_refFree(
markerlist = markerList[8:16],
spatial_count = spatial_count[1:2500, ],
spatial_location = spatial_location,
mincountgene = 100,
mincountspot = 5
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CARD.SCMapping.R
\name{get_weight_for_cell}
\alias{get_weight_for_cell}
\title{The function to estimate the cell type composition signature for each
single cell in the scRNaseq reference data}
\usage{
get_weight_for_cell(sc_eset, ct_varname, ct_select, sample_varname, B)
}
\arguments{
\item{sc_eset}{the sc_eset stored in the CARD object}

\item{ct_varname}{character, the name of the column in metaData that
specifies the cell type annotation information, stored in the CARD object}

\item{ct_select}{vector of cell type names that you are interested in to
deconvolute, default as NULL. stored in the CARD object}

\item{sample_varname}{character,the name of the column in metaData that
specifies the sample information. stored in the CARD object}

\item{B}{reference basis matrix stored in the CARD object.}
}
\value{
Returns a matrix of the cell type composition signature for each
single cell in the scRNaseq reference
}
\description{
The function to estimate the cell type composition signature for each
single cell in the scRNaseq reference data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{CARD_visualize_prop_2CT}
\alias{CARD_visualize_prop_2CT}
\title{Visualize the spatial distribution of two cell type proportions on
the same plot}
\usage{
CARD_visualize_prop_2CT(
  proportion,
  spatial_location,
  ct2_visualize = ct2_visualize,
  colors = NULL
)
}
\arguments{
\item{proportion}{Data frame, cell type proportion estimated by CARD in
either original resolution or enhanced resolution.}

\item{spatial_location}{Data frame, spatial location information.}

\item{ct2_visualize}{Vector of selected two cell type names that are
interested to visualize, here we only focus on two cell types}

\item{colors}{list of color names that you want to use for each cell type,
if NULL, we will use the default color scale list
list(c("lightblue","lightyellow","red"),c("lightblue","lightyellow","black")}
}
\value{
Returns a ggplot2 figure.
}
\description{
Visualize the spatial distribution of two cell type proportions on
the same plot
}
\examples{
library(ggplot2)
library(SpatialExperiment)
data(spatial_count)
data(spatial_location)
data(sc_count)
data(sc_meta)
CARD_obj <- CARD_deconvolution(
    sc_count = sc_count,
    sc_meta = sc_meta,
    spatial_count = spatial_count,
    spatial_location = spatial_location,
    ct_varname = "cellType",
    ct_select = unique(sc_meta$cellType),
    sample_varname = "sampleInfo",
    mincountgene = 100,
    mincountspot = 5
)
CARD_visualize_prop_2CT(
    proportion = CARD_obj$Proportion_CARD,
    spatial_location = spatialCoords(CARD_obj),
    ct2_visualize = c("Cancer_clone_A", "Cancer_clone_B"),
    colors = list(c("lightblue", "lightyellow", "red"), c(
        "lightblue", "lightyellow",
        "black"
    ))
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CARD.SCMapping.R
\name{CARD_scmapping}
\alias{CARD_scmapping}
\title{Extension of CARD into performing single cell Mapping from non-single
cell spatial transcriptomics dataset.}
\usage{
CARD_scmapping(CARD_object, shapeSpot = "Square", numcell, ncore = 10)
}
\arguments{
\item{CARD_object}{CARD object create by the CARD_deconvolution function.}

\item{shapeSpot}{a character indicating whether the sampled spatial
coordinates for single cells locating in a Square-like region or a
Circle-like region. The center of this region is the measured spatial
location in the non-single cell resolution spatial transcriptomics data.
The default is 'Square', the other shape is 'Circle'}

\item{numcell}{a numeric value indicating the number of single cells in
each measured location, we suggest 20 for ST technology, 7 for 10x Viisum
and 2 for Slide-seq}

\item{ncore}{a numeric value indicating the number of cores used to
accelerating the procedure}
}
\value{
Returns a SingleCellExperiment SCE object with the mapped expression
at single cell resolution and the spatial location information of each
single cell
}
\description{
Extension of CARD into performing single cell Mapping from non-single
cell spatial transcriptomics dataset.
}
\examples{
library(SingleCellExperiment)
data(spatial_count)
data(spatial_location)
data(sc_count)
data(sc_meta)
CARD_obj <- CARD_deconvolution(
    sc_count = sc_count,
    sc_meta = sc_meta,
    spatial_count = spatial_count,
    spatial_location = spatial_location,
    ct_varname = "cellType",
    ct_select = unique(sc_meta$cellType),
    sample_varname = "sampleInfo",
    mincountgene = 100,
    mincountspot = 5
)
scMapping <- CARD_scmapping(
CARD_obj, 
shapeSpot = "Square", 
numcell = 20, 
ncore = 2)
print(scMapping)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sc_meta.R
\docType{data}
\name{sc_meta}
\alias{sc_meta}
\title{scRNAseq meta data}
\format{
An object of class \code{data.frame} with 1926 rows and 3 columns.
}
\usage{
data(sc_meta)
}
\description{
The scRNAseq meta data must be in the format of data frame while each row
represents a cell. The rownames of the scRNAseq meta data should match
exactly with the column names of the scRNAseq count data. The sc_meta data
must contain the column indicating the cell type assignment for each cell
(e.g., “cellType” column in the example sc_meta data). Sample/subject
information should be provided, if there is only one sample, we can add a
column by sc_meta$sampleInfo = "sample1".
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CARD.imputation.R
\name{sample_grid_within}
\alias{sample_grid_within}
\title{Make new spatial locations on unmeasured tissue through grids.}
\usage{
sample_grid_within(location, num_sample, concavity = 2)
}
\arguments{
\item{location}{Data frame, spatial location data frame of the original
spatial resolved transcriptomics dataset, stored in the
spatialCoords(CARD_object)}

\item{num_sample}{Numeric, approximate number of cells in grid within the
shape of the spatial location data frame}

\item{concavity}{Numeric, a relative measure of concavity. The default is
2.0, which can prodecure detailed enough shapes. Infinity results in a
convex hull while 1 results in a more detailed shape.}
}
\value{
Return a list of data frame with newly grided points
}
\description{
Make new spatial locations on unmeasured tissue through grids.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CARD.prop.R
\name{select_info}
\alias{select_info}
\title{Select Informative Genes used in the deconvolution}
\usage{
select_info(basis, sc_eset, commongene, ct_select, ct_varname)
}
\arguments{
\item{basis}{Reference basis matrix.}

\item{sc_eset}{scRNAseq data along with meta data stored in the S4 class
format (SingleCellExperiment).}

\item{commongene}{common genes between scRNAseq count data and spatial
resolved transcriptomics data.}

\item{ct_select}{vector of cell type names that you are interested in to
deconvolute, default as NULL. If NULL, then use all cell types provided by
single cell dataset;}

\item{ct_varname}{character, the name of the column in metaData that
specifies the cell type annotation information}
}
\value{
a vector of informative genes selected
}
\description{
Select Informative Genes used in the deconvolution
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markerList.R
\docType{data}
\name{markerList}
\alias{markerList}
\title{marker gene list}
\format{
An object of class \code{list} of length 20.
}
\usage{
data(markerList)
}
\description{
The marker gene list is a list format with each element of the list being the
cell type specific gene markers.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{createCARDfreeObject}
\alias{createCARDfreeObject}
\title{Create the CARD object}
\usage{
createCARDfreeObject(
  markerlist,
  spatial_count,
  spatial_location,
  mincountgene = 100,
  mincountspot = 5,
  spe = NULL
)
}
\arguments{
\item{markerlist}{a list of marker genes, with each element of the list
being the vector of cell type specific marker genes}

\item{spatial_count}{Raw spatial resolved transcriptomics data, each column
is a spatial location, and each row is a gene.}

\item{spatial_location}{data frame, with two columns representing the x and
y coordinates of the spatial location. The rownames of this data frame
should match eaxctly with the columns of the spatial_count.}

\item{mincountgene}{Minimum counts for each gene}

\item{mincountspot}{Minimum counts for each spatial location}

\item{spe}{a \code{SpatialExperiment} object containing spatial data in
the \code{counts} assay, and spatial coordinates in the spatialCoords.}
}
\value{
Returns CARDfree object with filtered spatial count and marker gene
list.
}
\description{
Create the CARD object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{CARDref}
\alias{CARDref}
\title{SpatialDeconv function based on Conditional Autoregressive model}
\usage{
CARDref(
  XinputIn,
  UIn,
  WIn,
  phiIn,
  max_iterIn,
  epsilonIn,
  initV,
  initb,
  initSigma_e2,
  initLambda
)
}
\arguments{
\item{XinputIn}{The input of normalized spatial data}

\item{UIn}{The input of cell type specific basis matrix B}

\item{WIn}{The constructed W weight matrix from Gaussian kernel}

\item{phiIn}{The phi value}

\item{max_iterIn}{Maximum iterations}

\item{epsilonIn}{epsilon for convergence}

\item{initV}{Initial matrix of cell type compositions V}

\item{initb}{Initial vector of cell type specific intercept}

\item{initSigma_e2}{Initial value of residual variance}

\item{initLambda}{Initial vector of cell type sepcific scalar.}
}
\value{
A list
}
\description{
SpatialDeconv function based on Conditional Autoregressive model
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CARD.imputation.R
\name{CARD_imputation}
\alias{CARD_imputation}
\title{Construct an enhanced spatial expression map on the unmeasured tissue
locations}
\usage{
CARD_imputation(CARD_object, num_grids, ineibor = 10, exclude = NULL)
}
\arguments{
\item{CARD_object}{SpatialExperiment Object created by
CARD_deconvolution with estimated cell type compositions on the
original spatial resolved transcriptomics data.}

\item{num_grids}{Initial number of newly grided spatial locations. The final
number of newly grided spatial locations will be lower than this value since
the newly grided locations outside the shape of the tissue will be filtered}

\item{ineibor}{Numeric, number of neighbors used in the imputation on newly
grided spatial locations, default is 10.}

\item{exclude}{Vector, the rownames of spatial location data on the original
resolution that you want to exclude. This is to avoid the weird detection of
the shape.}
}
\value{
Return a SpatialExperiment object with the refined cell type
compositions estimated for newly grided spots and the refined predicted
gene expression (normalized).
}
\description{
Construct an enhanced spatial expression map on the unmeasured tissue
locations
}
\examples{
data(spatial_count)
data(spatial_location)
data(sc_count)
data(sc_meta)
CARD_obj <- CARD_deconvolution(
    sc_count = sc_count,
    sc_meta = sc_meta,
    spatial_count = spatial_count,
    spatial_location = spatial_location,
    ct_varname = "cellType",
    ct_select = unique(sc_meta$cellType),
    sample_varname = "sampleInfo",
    mincountgene = 100,
    mincountspot = 5
)
CARD_obj <- CARD_imputation(
    CARD_obj,
    num_grids = 200,
    ineibor = 10,
    exclude = NULL
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{CARD-class}
\alias{CARD-class}
\title{Each CARD object has a number of slots which store information. Key
slots to access are listed below.}
\value{
Return an object of CARD class
}
\description{
Each CARD object has a number of slots which store information. Key
slots to access are listed below.
}
\section{Slots}{

\describe{
\item{\code{sc_eset}}{The filtered scRNA-seq data along with meta data stored
in the format of SingleCellExperiment.}

\item{\code{spatial_countMat}}{The filtered spatial count data.}

\item{\code{spatial_location}}{The weights for combining p-values from multiple
kernels.}

\item{\code{Proportion_CARD}}{The estimated cell type proportion by
CARD with each row is a spatial location and each column is a
cell type.}

\item{\code{project}}{The name of the project, default is deconvolution.}

\item{\code{info_parameters}}{The paramters that are used in model fitting.}

\item{\code{algorithm_matrix}}{The intermediate matrices that are used in
the model fitting step.}

\item{\code{refined_prop}}{The refined cell type proportion matrix estimated
by CARD for the newly grided spatial locations. The number of
initial grids are defined by the user.}

\item{\code{refined_expression}}{The refined predicted expression matrix
(normalized) estimated by CARD for the newly grided spatial locations.
The number of initial grids are defined by the user.}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{show,CARD-method}
\alias{show,CARD-method}
\title{Show method for the CARD class}
\usage{
\S4method{show}{CARD}(object)
}
\arguments{
\item{object}{An object of class \code{CARD}.}
}
\value{
A concise summary of the \code{CARD} object is printed to the
console.
}
\description{
This method provides a concise summary of an object of class \code{CARD},
displaying key information including the project name, the number of spots,
the number of cell types, and a sample of the
\code{Proportion_CARD} matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CARD.imputation.R
\name{mvn_cv}
\alias{mvn_cv}
\title{Imputation and Construction of High-Resolution Spatial Maps for Cell Type
Composition and Gene Expression by the spatial correlation structure between
original spatial locations and new grided spatial locations}
\usage{
mvn_cv(
  vtrain,
  location_orig,
  train_ind,
  test_ind,
  B,
  xinput_norm,
  optimal_b,
  optimal_phi,
  lambda,
  ineibor
)
}
\arguments{
\item{vtrain}{Matrix, estimated V matrix from CARD}

\item{location_orig}{Data frame, spatial location data frame of the original
spatial resolved transcriptomics dataset, stored in the
spatialCoords(CARD_object)}

\item{train_ind}{Vector, index of the original spatial locations}

\item{test_ind}{Vector, index of the newly grided spatial locations}

\item{B}{Matrix, used in the deconvolution as the reference basis matrix}

\item{xinput_norm}{Matrix, used in the deconvolution as the normalized
spatial count data}

\item{optimal_b}{Vector, vector of the intercept for each cel type estimated
based on the original spatial resolution}

\item{optimal_phi}{Numeric, the optimal phi value stored in CARD_object}

\item{lambda}{Vector, vector of cell type specific scalar in the CAR model}

\item{ineibor}{Numeric, number of neighbors used in the imputation on newly
grided spatial locations, default is 10.}
}
\value{
Return a list with the imputed Cell type composition Vtest matrix on
the newly grided spatial locations and predicted normalized gene expression
}
\description{
Imputation and Construction of High-Resolution Spatial Maps for Cell Type
Composition and Gene Expression by the spatial correlation structure between
original spatial locations and new grided spatial locations
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CARD.prop.R
\name{create_ref}
\alias{create_ref}
\title{Construct the mean gene expression basis matrix (B), this is the
faster version}
\usage{
create_ref(sc_eset, ct_select = NULL, ct_varname, sample_varname = NULL)
}
\arguments{
\item{sc_eset}{S4 class for storing data from single-cell experiments.
This format is usually created by the package SingleCellExperiment with
stored counts, along with the usual metadata for genes and cells.}

\item{ct_select}{vector of cell type names that you are interested in to
deconvolute, default as NULL. If NULL, then use all cell types provided by
single cell dataset;}

\item{ct_varname}{character, the name of the column in metaData that
specifies the cell type annotation information}

\item{sample_varname}{character,the name of the column in metaData that
specifies the sample information. If NULL, we just use the whole as one
sample.}
}
\value{
Return a list of basis (B) matrix
}
\description{
Construct the mean gene expression basis matrix (B), this is the
faster version
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CARD.SCMapping.R
\name{get_high_res_cords}
\alias{get_high_res_cords}
\title{The function to sample the spatial location information for each single
cell}
\usage{
get_high_res_cords(cords, numcell, shape = "Square")
}
\arguments{
\item{cords}{The spatial location information in the measure spatial
locations, with the first and second columns represent the 2-D
x-y coordinate system}

\item{numcell}{a numeric value indicating the number of single cells in each
measured location, we suggest 20 for ST technology, 7 for 10x Viisum and 2
for Slide-seq}

\item{shape}{a character indicating whether the sampled spatial coordinates
for single cells locating in a Square-like region or a Circle-like region.
The center of this region is the measured spatial location in the non-single
cell resolution spatial transcriptomics data. The default is 'Square', the
other shape is 'Circle'}
}
\value{
Returns a dataframe with the sampled spatial location information
for each single cell
}
\description{
The function to sample the spatial location information for each single
cell
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CARD.prop.R
\name{CARD_deconvolution}
\alias{CARD_deconvolution}
\title{Spatially Informed Cell Type Deconvolution for Spatial Transcriptomics
by CARD}
\usage{
CARD_deconvolution(
  sc_count,
  sc_meta,
  spatial_count,
  spatial_location,
  ct_varname,
  ct_select,
  sample_varname,
  mincountgene = 100,
  mincountspot = 5,
  sce = NULL,
  spe = NULL
)
}
\arguments{
\item{sc_count}{Raw scRNA-seq count data, each column is a cell and each row
is a gene.}

\item{sc_meta}{data frame, with each row representing the cell type and/or
sample information of a specific cell. The row names of this data frame
should match exactly with the column names of the sc_count data}

\item{spatial_count}{Raw spatial resolved transcriptomics data, each column
is a spatial location, and each row is a gene.}

\item{spatial_location}{data frame, with two columns representing the x and
y coordinates of the spatial location. The rownames of this data frame
should match eaxctly with the columns of the spatial_count.}

\item{ct_varname}{character, the name of the column in metaData that
specifies the cell type annotation information}

\item{ct_select}{vector of cell type names that you are interested in to
deconvolute, default as NULL. If NULL, then use all cell types provided by
single cell dataset;}

\item{sample_varname}{character,the name of the column in metaData that
specifies the sample information. If NULL, we just use the whole as one
sample.}

\item{mincountgene}{Minimum counts for each gene}

\item{mincountspot}{Minimum counts for each spatial location}

\item{sce}{a \code{SingleCellExperiment} object containing scRNA-seq count
data in the \code{counts} assay, and cell types and sample information in
the colData.}

\item{spe}{a \code{SpatialExperiment} object containing spatial data in
the \code{counts} assay, and spatial coordinates in the spatialCoords.}
}
\value{
Returns a SpatialExperiment object with estimated cell type
proportion stored in object$Proportion_CARD.
}
\description{
Spatially Informed Cell Type Deconvolution for Spatial Transcriptomics
by CARD
}
\examples{
library(RcppML)
library(NMF)
library(RcppArmadillo)
data(spatial_count)
data(spatial_location)
data(sc_count)
data(sc_meta)
CARD_obj <- CARD_deconvolution(
    sc_count = sc_count,
    sc_meta = sc_meta,
    spatial_count = spatial_count,
    spatial_location = spatial_location,
    ct_varname = "cellType",
    ct_select = unique(sc_meta$cellType),
    sample_varname = "sampleInfo",
    mincountgene = 100,
    mincountspot = 5
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{CARDfree-class}
\alias{CARDfree-class}
\title{Each CARDfree object has a number of slots which store information.
Key slots to access are listed below.}
\value{
Return an object of CARDfree class
}
\description{
Each CARDfree object has a number of slots which store information.
Key slots to access are listed below.
}
\section{Slots}{

\describe{
\item{\code{spatial_countMat}}{The filtered spatial count data.}

\item{\code{spatial_location}}{The weights for combining p-values from multiple
kernels.}

\item{\code{Proportion_CARD}}{The estimated cell type proportion by CARD
with each row is a spatial location and each column is a cell type.}

\item{\code{estimated_refMatrix}}{The estimated reference matrix by CARDfree
with each row represents a gene and each column represents a cell type
cluster.}

\item{\code{project}}{The name of the project, default is deconvolution.}

\item{\code{markerList}}{The nlist of cell type specific markers, with
each element represents the vector of cell type specific markers}

\item{\code{info_parameters}}{The paramters that are used in model fitting.}

\item{\code{algorithm_matrix}}{The intermediate matrices that are used in
the model fitting step.}

\item{\code{refined_prop}}{The refined cell type proportion matrix estimated
by CARD for the newly grided spatial locations. The number of initial
grids are defined by the user.}

\item{\code{refined_expression}}{The refined predicted expression matrix
(normalized) estimated by CARD for the newly grided spatial locations.
The number of initial grids are defined by the user.}
}}
